##################### generated by xml-casa (v2) from delmod.xml ####################
##################### 2f0bbe58ff0653115c12b5dc327582ae ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from casatasks.private.task_delmod import delmod as _delmod_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _delmod:
    """
    delmod ---- Deletes model representations in the MS

    
    This utility task is to be used to delete the model visibility data
    representations in the MS.

    --------- parameter descriptions ---------------------------------------------

    vis     Name of input visibility file (MS)
    otf     Delete the on-the-fly model data keywords
            Default=True
            
            The 'otf' representation is the 'scratch-less'
            model data, stored as keywords in the MS header
            containing model data formation instructions.  It
            is generated by the setjy, ft, and tclean tasks
            (usescratch=False), and if present, overrides the
            MODEL_DATA column (if present). If a user wishes
            to use the MODEL_DATA column _after_ having
            operated with the 'otf' representation, this task
            can be used to delete the 'otf' represenatation
            to make the MODEL_DATA column visible.  (Create
            the MODEL_DATA column by using usescratch=True in
            setjy, ft, or clean; or by running the clearcal
            task with addmodel=True.)
    field   Select field using field id(s) or field name(s)
            Subparameter of otf
            Default: '' (all fields' models will be deleted)
    scr     Delete the MODEL_DATA scr col (if it exists)
            Default: False
            
            Note: it is not possible to delete the MODEL_DATA
            column per field.
            
            If otf=F and scr=F, delmod will provide a listing
            of the header field records.
    [1;42mRETURNS[1;m    void

    --------- examples -----------------------------------------------------------

    
    
    
    For more information, see the task pages of delmod in CASA Docs:
    
    https://casa.nrao.edu/casadocs/
    
    


    """

    _info_group_ = """imaging, calibration"""
    _info_desc_ = """Deletes model representations in the MS"""

    __schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'otf': {'type': 'cBool'}, 'field': {'type': 'cStr', 'coerce': _coerce.to_str}, 'scr': {'type': 'cBool'}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 8 + 4
        output = [ ]
        addon = ''
        first_addon = True
        if len(description) == 0:
            out.write(param_prefix + " #\n")
            return
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __vis_dflt( self, glb ):
        return ''

    def __vis( self, glb ):
        if 'vis' in glb: return glb['vis']
        return ''

    def __otf_dflt( self, glb ):
        return True

    def __otf( self, glb ):
        if 'otf' in glb: return glb['otf']
        return True

    def __scr_dflt( self, glb ):
        return False

    def __scr( self, glb ):
        if 'scr' in glb: return glb['scr']
        return False



    #--------- return inp/go default --------------------------------------------------
    def __field_dflt( self, glb ):
        if self.__otf( glb ) == bool(True): return ""
        return None

    #--------- return subparam values -------------------------------------------------
    def __field( self, glb ):
        if 'field' in glb: return glb['field']
        dflt = self.__field_dflt( glb )
        if dflt is not None: return dflt
        return ''

    #--------- subparam inp output ----------------------------------------------------
    def __vis_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__vis_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Name of input visibility file (MS)'
        value = self.__vis( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'vis': value},{'vis': self.__schema['vis']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-8.8s = %s%-23s%s' % ('vis',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __otf_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__otf_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return True
        description = 'Delete the on-the-fly model data keywords'
        value = self.__otf( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'otf': value},{'otf': self.__schema['otf']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-8.8s =\x1B[0m %s%-23s%s' % ('otf',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __field_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__field_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__field_dflt( self.__globals_( ) ) is not None:
             description = 'Select field using field id(s) or field name(s)'
             value = self.__field( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'field': value},{'field': self.__schema['field']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-5.5s =\x1B[0m %s%-23s%s' % ('field',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __scr_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__scr_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Delete the MODEL_DATA scr col (if it exists)'
        value = self.__scr( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'scr': value},{'scr': self.__schema['scr']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-8.8s = %s%-23s%s' % ('scr',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'vis' in glb: del glb['vis']
        if 'otf' in glb: del glb['otf']
        if 'field' in glb: del glb['field']
        if 'scr' in glb: del glb['scr']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# delmod -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__vis_inp( )
        self.__otf_inp( )
        self.__field_inp( )
        self.__scr_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,savefile=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = savefile
        if filename is None:
            filename = "delmod.last" if os.path.isfile("delmod.last") else "delmod.saved"
        if os.path.isfile(filename):
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file: %s\nsetting defaults instead..." % filename)
            self.set_global_defaults( )

    #--------- tput function ----------------------------------------------------------
    def tput(self,outfile=None):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s

        _postfile = outfile if outfile is not None else os.path.realpath('delmod.last')

        _invocation_parameters = OrderedDict( )
        _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
        _invocation_parameters['otf'] = self.__otf( self.__globals_( ) )
        _invocation_parameters['field'] = self.__field( self.__globals_( ) )
        _invocation_parameters['scr'] = self.__scr( self.__globals_( ) )

        try:
            with open(_postfile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-5s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#delmod( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: return False
        return True

    def __call__( self, vis=None, otf=None, field=None, scr=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('delmod.pre')
        _postfile = os.path.realpath('delmod.last')
        task_result = None
        _arguments = [vis,otf,field,scr]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if vis is not None: local_global['vis'] = vis
            if otf is not None: local_global['otf'] = otf
            if scr is not None: local_global['scr'] = scr

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['vis'] = self.__vis( local_global )
            _invocation_parameters['otf'] = self.__otf( local_global )
            _invocation_parameters['scr'] = self.__scr( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            _invocation_parameters['field'] = self.__field( _invocation_parameters ) if field is None else field

        else:
            # invoke with inp/go semantics
            _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
            _invocation_parameters['otf'] = self.__otf( self.__globals_( ) )
            _invocation_parameters['field'] = self.__field( self.__globals_( ) )
            _invocation_parameters['scr'] = self.__scr( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-5s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#delmod( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _logging_state_ = None
            assert _pc.validate(_invocation_parameters,self.__schema), create_error_string(_pc.errors)
            _logging_state_ = _start_log( 'delmod', [ 'vis=' + repr(_pc.document['vis']), 'otf=' + repr(_pc.document['otf']), 'field=' + repr(_pc.document['field']), 'scr=' + repr(_pc.document['scr']) ] )
            task_result = _delmod_t( _pc.document['vis'],_pc.document['otf'],_pc.document['field'],_pc.document['scr'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('delmod')
            casalog.post("Exception Reported: Error in delmod: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            raise #exception is now raised
            #task_result = False
        finally:
            try:
                os.rename(_prefile,_postfile)
            except: pass
            if _logging_state_:
                task_result = _end_log( _logging_state_, 'delmod', task_result )

        #Added if _delmod_t returns False and does not raise an exception.
        if task_result is False:
            raise

        return task_result #Still needed

delmod = _delmod( )

