##################### generated by xml-casa (v2) from flagcmd.xml ###################
##################### b8a2611ec7c82cf7a20bc5e22bf736d2 ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from casatasks.private.task_flagcmd import flagcmd as _flagcmd_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _flagcmd:
    """
    flagcmd ---- Flagging task based on batches of flag-commands

    
    The flagcmd task will flag the visibility data or calibration
    table based on several batch-operations using flag commands.
    
    Flag commands follow the mode and parameter names from the
    flagdata task.
    
    The flagcmd task will flag data based on the commands input on
    inpmode:
    table = input from FLAG_CMD table in MS
    list  = input from text file or list of strings from inpfile
    xml   = input from Flag.xml in the MS given by vis
    
    Batch operations include : apply/unapply/list/plot/clear/extract
    
    IMPORTANT: If you use other ways to flag such as interactive
    flagging in plotms, the FLAG_CMD will NOT be updated!
    
    NOTE on flagging calibration tables.
    -----------------------------------
    We recommend using the flagdata task for flagging cal tabels. When
    using flagcmd to flag cal tables, only the 'apply' and 'list'
    actions are supported. Because cal tables do not have a FLAG_CMD
    sub-table, the default inpmode='table' can only be used if an MS
    is given in the 'inpfile' parameter so that flags from the MS are
    applied to the cal table. Otherwise, the flag commands must be
    given using inpmode='list', either from a file(s) or from a list
    of strings. Data selection for calibration tables is limited to
    field, scan, antenna, time, spw and observation.
    
    

    --------- parameter descriptions ---------------------------------------------

    vis        Name of input visibility file or calibration table.
               default: '' (none) 
               
                  example: vis='uid___A002_X2a5c2f_X54.ms'
    inpmode    Input mode for flag commands(table/list/xml)
               options: 'table','list','xml'
               default: 'table' (the input commands from
               FLAG_CMD table of the MS)
               
               inpmode='xml' inputs online flags from Flag.xml
               file in the MS. This mode has become largely
               obsolete with the deprecation of the importevla
               task (see the flagcmd task pages in CASA Docs for
               more information). This mode will not work for
               ALMA MS or cal tables.
               
               NOTE: You can only apply the flags from a list or
               xml; you will not be able to unapply
               them. Transfer the flag commands to the FLAG_CMD
               table if you want to unapply the flags (see
               'inpfile' description below).
    inpfile    Source of flag commands. Subparameter of
               inpmode='table/list'.
               Path to MS containing FLAG_CMD (table), or name
               of an ASCII file, list of files or a list of
               Python strings to apply to MS or cal table
               (list). 
               options: [] with flag commands or [] with
               filenames or '' with a filename. (String values
               must contain quotes around them or the parser
               will not work.)
               default: '' (read from FLAG_CMD table in the MS
               specified via 'vis')
               
               Main use is to read flags from internal FLAG_CMD,
               but one use case is to read the flag commands
               from an MS given in inpfile and apply them to
               another MS or cal table given in vis.
    tablerows  List of rows of the FLAG_CMD table to read. Subparameter
               of inpmode='table/list'.
               default: [] (read all rows)
               
                  example: [0,1,2,10]
               
               NOTE: currently only takes integer lists, not
               parseable strings with ranges.  Use the Python
               range function to generate ranges, e.g. tablerows
               = range(0,30) + range(50,55) instead of
               '0~29,50~54' for now.
    reason     Select flag commands based on REASON(s). Subparameter of
               inpmode.
               default: 'any' (all flags regardless of reason)
               
                  Examples: 
                  reason='FOCUS_ERROR'
                  reason=['FOCUS_ERROR','SUBREFLECTOR_ERROR']
               
               If inpfile is a list of files, the reasons given
               in this parameter will apply to all the files.
               
               NOTE: what is within the string is literally
               matched, e.g. reason='' matches only blank
               reasons, and reason
               ='FOCUS_ERROR,SUBREFLECTOR_ERROR' matches this
               compound reason string only
    useapplied Select commands whose rows have APPLIED column set to
               True. Subparameter of inpmode='table'.
               options: True,False
               default: False   
               
               If useapplied=True it will read in both applied
               and unapplied flags.
               
               IMPORTANT: The APPLIED column is set to True
               after a flag command is applied to the MS. In
               order to re-apply the same flag command, this
               parameter should be set to True.
    tbuff      Time buffer (sec) to pad flags. Subparameter of
               inpmode='xml'.
               default: 0.0
    ants       Allowed flag antenna names to select by. Subparameter of
               inpmode='xml'.
    action     Action to perform in MS and/or in inpfile
                 options: apply/unapply/list/plot/clear/extract
                 default: 'apply'
               
                    Examples:
                    -- action='apply': This operation will apply
                    the commands chosen by inpmode. If
                    inpmode='table' and inpfile='' then the
                    APPLIED column in FLAG_CMD will be set to
                    True.
                    -- action='unapply': unapply flags in MS. (Not
                    available for cal tables). This operation will
                    unapply the commands chosen by inpmode='table'
                    ONLY. After unapplying the commands, the task
                    will update the APPLIED column to False.
                    -- action='list': list and/or save flag
                    commands. This operation will list the
                    commands chosen by inpmode on the screen and
                    save them to the MS or to a file without
                    applying. It will save the commands to outfile
                    if the parameter savepars is set to True. If
                    outfile is None, it will save the commands to
                    the MS given in 'vis'.
                    -- action='plot': plot flags (ant
                    vs. time). (Not available for cal
                    tables). This operation will plot the flags
                    chosen by inpmode to a matplotlib gui or to a
                    file.  These will be sorted by antenna
                    vs. time.  Most useful for showing the online
                    flags.
                    -- action='clear': clear flags from FLAG_CMD
                    in the MS. (Not available for cal tables) This
                    operation will delete the selected flag rows
                    from the internal FLAG_CMD table of the MS.
                    -- action='extract': extract internal flag
                    dictionary. (Not available for cal tables)
                    This option will return the internal flagging
                    dictionary to python. There is no extant
                    description of the format of this dictionary,
                    as it is an internal device used by the
                    flagcmd task. This action is provided for the
                    convenience of advanced users.
               
                WARNING: choosing this action='clear' will
                disregard anything you set in inpmode and will
                always work on the FLAG_CMD table in vis. This can
                be used to totally delete rows from the FLAG_CMD
                table, when setting clearall=True.
    flagbackup Automatically backup the FLAG column before
               execution. Subparameter of action='apply/unapply'.
               options: True,False
               default: True
    clearall   Delete all rows from FLAG_CMD. Subparameter of
               action='clear'.
               default: False (will not clear)
    rowlist    FLAG_CMD rows to clear. Subparameter of action='clear'.
                     default: [] (all flags in table)
               
                        example: [0,1,2,10]
               
                     WARNING: this can be dangerous, and you must set
                     clearall=True  to use this!!! This will delete
                     the specified rows from the internal FLAG_CMD
                     table for vis regardless of what mode is set to
                     (useful for when you import from xml or file),
                     and decide to redo it). This action will NOT
                     unapply the commands.
               
                     NOTE: currently only takes integer lists, not
                     parseable strings with ranges.  Use the Python
                     range function to generate ranges, e.g. rowlist =
                     range(0,30) + range(50,55) instead of
                     '0~29,50~54' for now.
    plotfile   Name of output file to save plot
               default: '' (plot to matplotlib window)
               
               WARNING: will only reliably plot individual flags
               per antenna and timerange (e.g. direct from xml)
    savepars   Save the flag commands to the FLAG_CMD table of the MS or
               to an output text file.
               options: True/False     
               default: False
    outfile    Name of output file to save commands. Subparameter of
               savepars=True.
               default: ' '; it will save the commands in the
               FLAG_CMD table of the MS.
               
                  example: outfile='flags.txt' will save the
                  parameters in a text file.
    overwrite  Overwrite an existing file given in 'outfile' to save the
               flag commands. Subparameter of savepars=True.
               options: True/False
               default: True; it will remove the existing file
               given in 'outfile' and save the current flag
               commands to a new file with the same name. When
               set to False, the task will exit with an error
               message if the file exist.
    [1;42mRETURNS[1;m       void

    --------- examples -----------------------------------------------------------

    
    
    For more information, see the task pages of flagcmd in CASA Docs:
    
    https://casa.nrao.edu/casadocs/


    """

    _info_group_ = """flagging"""
    _info_desc_ = """Flagging task based on batches of flag-commands"""

    __schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'inpmode': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'table', 'list', 'xml' ]}, 'inpfile': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}]}, 'tablerows': {'type': 'cIntVec', 'coerce': [_coerce.to_list,_coerce.to_intvec]}, 'reason': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}]}, 'useapplied': {'type': 'cBool'}, 'tbuff': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'ants': {'type': 'cStr', 'coerce': _coerce.to_str}, 'action': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'apply', 'clear', 'plot', 'list', 'extract', 'unapply' ]}, 'flagbackup': {'type': 'cBool'}, 'clearall': {'type': 'cBool'}, 'rowlist': {'type': 'cIntVec', 'coerce': [_coerce.to_list,_coerce.to_intvec]}, 'plotfile': {'type': 'cStr', 'coerce': _coerce.to_str}, 'savepars': {'type': 'cBool'}, 'outfile': {'type': 'cStr', 'coerce': _coerce.to_str}, 'overwrite': {'type': 'cBool'}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 13 + 4
        output = [ ]
        addon = ''
        first_addon = True
        if len(description) == 0:
            out.write(param_prefix + " #\n")
            return
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __vis_dflt( self, glb ):
        return ''

    def __vis( self, glb ):
        if 'vis' in glb: return glb['vis']
        return ''

    def __inpmode_dflt( self, glb ):
        return 'table'

    def __inpmode( self, glb ):
        if 'inpmode' in glb: return glb['inpmode']
        return 'table'

    def __savepars_dflt( self, glb ):
        return False

    def __savepars( self, glb ):
        if 'savepars' in glb: return glb['savepars']
        return False

    def __action_dflt( self, glb ):
        return 'apply'

    def __action( self, glb ):
        if 'action' in glb: return glb['action']
        return 'apply'



    #--------- return inp/go default --------------------------------------------------
    def __inpfile_dflt( self, glb ):
        if self.__inpmode( glb ) == "table": return ""
        if self.__inpmode( glb ) == "list": return ""
        return None
    def __outfile_dflt( self, glb ):
        if self.__savepars( glb ) == bool(True): return ""
        return None
    def __rowlist_dflt( self, glb ):
        if self.__action( glb ) == "clear": return []
        return None
    def __useapplied_dflt( self, glb ):
        if self.__inpmode( glb ) == "table": return bool(False)
        return None
    def __clearall_dflt( self, glb ):
        if self.__action( glb ) == "clear": return bool(False)
        return None
    def __reason_dflt( self, glb ):
        if self.__inpmode( glb ) == "table": return "any"
        if self.__inpmode( glb ) == "list": return "any"
        if self.__inpmode( glb ) == "xml": return "any"
        return None
    def __tablerows_dflt( self, glb ):
        if self.__inpmode( glb ) == "table": return []
        return None
    def __plotfile_dflt( self, glb ):
        if self.__action( glb ) == "plot": return ""
        return None
    def __ants_dflt( self, glb ):
        if self.__inpmode( glb ) == "xml": return ""
        return None
    def __overwrite_dflt( self, glb ):
        if self.__savepars( glb ) == bool(True): return bool(True)
        return None
    def __tbuff_dflt( self, glb ):
        if self.__inpmode( glb ) == "xml": return float(0.0)
        return None
    def __flagbackup_dflt( self, glb ):
        if self.__action( glb ) == "apply": return bool(True)
        if self.__action( glb ) == "unapply": return bool(True)
        return None

    #--------- return subparam values -------------------------------------------------
    def __inpfile( self, glb ):
        if 'inpfile' in glb: return glb['inpfile']
        dflt = self.__inpfile_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __tablerows( self, glb ):
        if 'tablerows' in glb: return glb['tablerows']
        dflt = self.__tablerows_dflt( glb )
        if dflt is not None: return dflt
        return [  ]
    def __reason( self, glb ):
        if 'reason' in glb: return glb['reason']
        dflt = self.__reason_dflt( glb )
        if dflt is not None: return dflt
        return 'any'
    def __useapplied( self, glb ):
        if 'useapplied' in glb: return glb['useapplied']
        dflt = self.__useapplied_dflt( glb )
        if dflt is not None: return dflt
        return False
    def __tbuff( self, glb ):
        if 'tbuff' in glb: return glb['tbuff']
        dflt = self.__tbuff_dflt( glb )
        if dflt is not None: return dflt
        return float(0.0)
    def __ants( self, glb ):
        if 'ants' in glb: return glb['ants']
        dflt = self.__ants_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __flagbackup( self, glb ):
        if 'flagbackup' in glb: return glb['flagbackup']
        dflt = self.__flagbackup_dflt( glb )
        if dflt is not None: return dflt
        return True
    def __clearall( self, glb ):
        if 'clearall' in glb: return glb['clearall']
        dflt = self.__clearall_dflt( glb )
        if dflt is not None: return dflt
        return False
    def __rowlist( self, glb ):
        if 'rowlist' in glb: return glb['rowlist']
        dflt = self.__rowlist_dflt( glb )
        if dflt is not None: return dflt
        return [  ]
    def __plotfile( self, glb ):
        if 'plotfile' in glb: return glb['plotfile']
        dflt = self.__plotfile_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __outfile( self, glb ):
        if 'outfile' in glb: return glb['outfile']
        dflt = self.__outfile_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __overwrite( self, glb ):
        if 'overwrite' in glb: return glb['overwrite']
        dflt = self.__overwrite_dflt( glb )
        if dflt is not None: return dflt
        return True

    #--------- subparam inp output ----------------------------------------------------
    def __vis_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__vis_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Name of MS file or calibration table to flag'
        value = self.__vis( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'vis': value},{'vis': self.__schema['vis']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('vis',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __inpmode_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__inpmode_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'table'
        description = 'Input mode for flag commands(table/list/xml)'
        value = self.__inpmode( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'inpmode': value},{'inpmode': self.__schema['inpmode']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-13.13s =\x1B[0m %s%-23s%s' % ('inpmode',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __inpfile_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__inpfile_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__inpfile_dflt( self.__globals_( ) ) is not None:
             description = 'Source of flag commands'
             value = self.__inpfile( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'inpfile': value},{'inpfile': self.__schema['inpfile']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('inpfile',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __tablerows_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__tablerows_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return [  ]
        if self.__tablerows_dflt( self.__globals_( ) ) is not None:
             description = 'Rows of inpfile to read'
             value = self.__tablerows( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'tablerows': value},{'tablerows': self.__schema['tablerows']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('tablerows',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __reason_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__reason_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'any'
        if self.__reason_dflt( self.__globals_( ) ) is not None:
             description = 'Select by REASON types'
             value = self.__reason( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'reason': value},{'reason': self.__schema['reason']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('reason',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __useapplied_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__useapplied_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        if self.__useapplied_dflt( self.__globals_( ) ) is not None:
             description = 'Select commands whose rows have APPLIED column set to True'
             value = self.__useapplied( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'useapplied': value},{'useapplied': self.__schema['useapplied']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('useapplied',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __tbuff_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__tbuff_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return float(0.0)
        if self.__tbuff_dflt( self.__globals_( ) ) is not None:
             description = 'Time buffer (sec) to pad flags'
             value = self.__tbuff( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'tbuff': value},{'tbuff': self.__schema['tbuff']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('tbuff',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __ants_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__ants_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__ants_dflt( self.__globals_( ) ) is not None:
             description = 'Allowed flag antenna names to select by'
             value = self.__ants( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'ants': value},{'ants': self.__schema['ants']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('ants',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __action_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__action_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'apply'
        description = 'Action to perform in MS and/or in inpfile (apply/unapply/list/plot/clear/extract)'
        value = self.__action( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'action': value},{'action': self.__schema['action']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-13.13s =\x1B[0m %s%-23s%s' % ('action',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __flagbackup_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__flagbackup_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return True
        if self.__flagbackup_dflt( self.__globals_( ) ) is not None:
             description = 'Automatically backup the FLAG column before execution'
             value = self.__flagbackup( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'flagbackup': value},{'flagbackup': self.__schema['flagbackup']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('flagbackup',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __clearall_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__clearall_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        if self.__clearall_dflt( self.__globals_( ) ) is not None:
             description = 'Delete all rows from FLAG_CMD'
             value = self.__clearall( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'clearall': value},{'clearall': self.__schema['clearall']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('clearall',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __rowlist_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__rowlist_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return [  ]
        if self.__rowlist_dflt( self.__globals_( ) ) is not None:
             description = 'FLAG_CMD rows to clear'
             value = self.__rowlist( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'rowlist': value},{'rowlist': self.__schema['rowlist']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('rowlist',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __plotfile_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__plotfile_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__plotfile_dflt( self.__globals_( ) ) is not None:
             description = 'Name of output file to save plot'
             value = self.__plotfile( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'plotfile': value},{'plotfile': self.__schema['plotfile']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('plotfile',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __savepars_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__savepars_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Save flag commands to the MS or file'
        value = self.__savepars( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'savepars': value},{'savepars': self.__schema['savepars']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-13.13s =\x1B[0m %s%-23s%s' % ('savepars',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __outfile_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__outfile_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__outfile_dflt( self.__globals_( ) ) is not None:
             description = 'Name of output file to save commands'
             value = self.__outfile( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'outfile': value},{'outfile': self.__schema['outfile']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('outfile',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __overwrite_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__overwrite_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return True
        if self.__overwrite_dflt( self.__globals_( ) ) is not None:
             description = 'Overwrite an existing file to save the flag commands'
             value = self.__overwrite( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'overwrite': value},{'overwrite': self.__schema['overwrite']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-10.10s =\x1B[0m %s%-23s%s' % ('overwrite',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'inpfile' in glb: del glb['inpfile']
        if 'outfile' in glb: del glb['outfile']
        if 'rowlist' in glb: del glb['rowlist']
        if 'inpmode' in glb: del glb['inpmode']
        if 'useapplied' in glb: del glb['useapplied']
        if 'clearall' in glb: del glb['clearall']
        if 'vis' in glb: del glb['vis']
        if 'reason' in glb: del glb['reason']
        if 'tablerows' in glb: del glb['tablerows']
        if 'plotfile' in glb: del glb['plotfile']
        if 'ants' in glb: del glb['ants']
        if 'overwrite' in glb: del glb['overwrite']
        if 'tbuff' in glb: del glb['tbuff']
        if 'flagbackup' in glb: del glb['flagbackup']
        if 'savepars' in glb: del glb['savepars']
        if 'action' in glb: del glb['action']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# flagcmd -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__vis_inp( )
        self.__inpmode_inp( )
        self.__inpfile_inp( )
        self.__tablerows_inp( )
        self.__reason_inp( )
        self.__useapplied_inp( )
        self.__tbuff_inp( )
        self.__ants_inp( )
        self.__action_inp( )
        self.__flagbackup_inp( )
        self.__clearall_inp( )
        self.__rowlist_inp( )
        self.__plotfile_inp( )
        self.__savepars_inp( )
        self.__outfile_inp( )
        self.__overwrite_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,savefile=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = savefile
        if filename is None:
            filename = "flagcmd.last" if os.path.isfile("flagcmd.last") else "flagcmd.saved"
        if os.path.isfile(filename):
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file: %s\nsetting defaults instead..." % filename)
            self.set_global_defaults( )

    #--------- tput function ----------------------------------------------------------
    def tput(self,outfile=None):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s

        _postfile = outfile if outfile is not None else os.path.realpath('flagcmd.last')

        _invocation_parameters = OrderedDict( )
        _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
        _invocation_parameters['inpmode'] = self.__inpmode( self.__globals_( ) )
        _invocation_parameters['inpfile'] = self.__inpfile( self.__globals_( ) )
        _invocation_parameters['tablerows'] = self.__tablerows( self.__globals_( ) )
        _invocation_parameters['reason'] = self.__reason( self.__globals_( ) )
        _invocation_parameters['useapplied'] = self.__useapplied( self.__globals_( ) )
        _invocation_parameters['tbuff'] = self.__tbuff( self.__globals_( ) )
        _invocation_parameters['ants'] = self.__ants( self.__globals_( ) )
        _invocation_parameters['action'] = self.__action( self.__globals_( ) )
        _invocation_parameters['flagbackup'] = self.__flagbackup( self.__globals_( ) )
        _invocation_parameters['clearall'] = self.__clearall( self.__globals_( ) )
        _invocation_parameters['rowlist'] = self.__rowlist( self.__globals_( ) )
        _invocation_parameters['plotfile'] = self.__plotfile( self.__globals_( ) )
        _invocation_parameters['savepars'] = self.__savepars( self.__globals_( ) )
        _invocation_parameters['outfile'] = self.__outfile( self.__globals_( ) )
        _invocation_parameters['overwrite'] = self.__overwrite( self.__globals_( ) )

        try:
            with open(_postfile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-10s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#flagcmd( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: return False
        return True

    def __call__( self, vis=None, inpmode=None, inpfile=None, tablerows=None, reason=None, useapplied=None, tbuff=None, ants=None, action=None, flagbackup=None, clearall=None, rowlist=None, plotfile=None, savepars=None, outfile=None, overwrite=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('flagcmd.pre')
        _postfile = os.path.realpath('flagcmd.last')
        task_result = None
        _arguments = [vis,inpmode,inpfile,tablerows,reason,useapplied,tbuff,ants,action,flagbackup,clearall,rowlist,plotfile,savepars,outfile,overwrite]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if vis is not None: local_global['vis'] = vis
            if inpmode is not None: local_global['inpmode'] = inpmode
            if action is not None: local_global['action'] = action
            if savepars is not None: local_global['savepars'] = savepars

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['vis'] = self.__vis( local_global )
            _invocation_parameters['inpmode'] = self.__inpmode( local_global )
            _invocation_parameters['action'] = self.__action( local_global )
            _invocation_parameters['savepars'] = self.__savepars( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            _invocation_parameters['inpfile'] = self.__inpfile( _invocation_parameters ) if inpfile is None else inpfile
            _invocation_parameters['tablerows'] = self.__tablerows( _invocation_parameters ) if tablerows is None else tablerows
            _invocation_parameters['reason'] = self.__reason( _invocation_parameters ) if reason is None else reason
            _invocation_parameters['useapplied'] = self.__useapplied( _invocation_parameters ) if useapplied is None else useapplied
            _invocation_parameters['tbuff'] = self.__tbuff( _invocation_parameters ) if tbuff is None else tbuff
            _invocation_parameters['ants'] = self.__ants( _invocation_parameters ) if ants is None else ants
            _invocation_parameters['flagbackup'] = self.__flagbackup( _invocation_parameters ) if flagbackup is None else flagbackup
            _invocation_parameters['clearall'] = self.__clearall( _invocation_parameters ) if clearall is None else clearall
            _invocation_parameters['rowlist'] = self.__rowlist( _invocation_parameters ) if rowlist is None else rowlist
            _invocation_parameters['plotfile'] = self.__plotfile( _invocation_parameters ) if plotfile is None else plotfile
            _invocation_parameters['outfile'] = self.__outfile( _invocation_parameters ) if outfile is None else outfile
            _invocation_parameters['overwrite'] = self.__overwrite( _invocation_parameters ) if overwrite is None else overwrite

        else:
            # invoke with inp/go semantics
            _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
            _invocation_parameters['inpmode'] = self.__inpmode( self.__globals_( ) )
            _invocation_parameters['inpfile'] = self.__inpfile( self.__globals_( ) )
            _invocation_parameters['tablerows'] = self.__tablerows( self.__globals_( ) )
            _invocation_parameters['reason'] = self.__reason( self.__globals_( ) )
            _invocation_parameters['useapplied'] = self.__useapplied( self.__globals_( ) )
            _invocation_parameters['tbuff'] = self.__tbuff( self.__globals_( ) )
            _invocation_parameters['ants'] = self.__ants( self.__globals_( ) )
            _invocation_parameters['action'] = self.__action( self.__globals_( ) )
            _invocation_parameters['flagbackup'] = self.__flagbackup( self.__globals_( ) )
            _invocation_parameters['clearall'] = self.__clearall( self.__globals_( ) )
            _invocation_parameters['rowlist'] = self.__rowlist( self.__globals_( ) )
            _invocation_parameters['plotfile'] = self.__plotfile( self.__globals_( ) )
            _invocation_parameters['savepars'] = self.__savepars( self.__globals_( ) )
            _invocation_parameters['outfile'] = self.__outfile( self.__globals_( ) )
            _invocation_parameters['overwrite'] = self.__overwrite( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-10s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#flagcmd( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _logging_state_ = None
            assert _pc.validate(_invocation_parameters,self.__schema), create_error_string(_pc.errors)
            _logging_state_ = _start_log( 'flagcmd', [ 'vis=' + repr(_pc.document['vis']), 'inpmode=' + repr(_pc.document['inpmode']), 'inpfile=' + repr(_pc.document['inpfile']), 'tablerows=' + repr(_pc.document['tablerows']), 'reason=' + repr(_pc.document['reason']), 'useapplied=' + repr(_pc.document['useapplied']), 'tbuff=' + repr(_pc.document['tbuff']), 'ants=' + repr(_pc.document['ants']), 'action=' + repr(_pc.document['action']), 'flagbackup=' + repr(_pc.document['flagbackup']), 'clearall=' + repr(_pc.document['clearall']), 'rowlist=' + repr(_pc.document['rowlist']), 'plotfile=' + repr(_pc.document['plotfile']), 'savepars=' + repr(_pc.document['savepars']), 'outfile=' + repr(_pc.document['outfile']), 'overwrite=' + repr(_pc.document['overwrite']) ] )
            task_result = _flagcmd_t( _pc.document['vis'],_pc.document['inpmode'],_pc.document['inpfile'],_pc.document['tablerows'],_pc.document['reason'],_pc.document['useapplied'],_pc.document['tbuff'],_pc.document['ants'],_pc.document['action'],_pc.document['flagbackup'],_pc.document['clearall'],_pc.document['rowlist'],_pc.document['plotfile'],_pc.document['savepars'],_pc.document['outfile'],_pc.document['overwrite'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('flagcmd')
            casalog.post("Exception Reported: Error in flagcmd: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            raise #exception is now raised
            #task_result = False
        finally:
            try:
                os.rename(_prefile,_postfile)
            except: pass
            if _logging_state_:
                task_result = _end_log( _logging_state_, 'flagcmd', task_result )

        #Added if _flagcmd_t returns False and does not raise an exception.
        if task_result is False:
            raise

        return task_result #Still needed

flagcmd = _flagcmd( )

