##################### generated by xml-casa (v2) from importfitsidi.xml #############
##################### fedf829bc2603c34a92fe21c2ceaee4a ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from casatasks.private.task_importfitsidi import importfitsidi as _importfitsidi_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _importfitsidi:
    """
    importfitsidi ---- Convert a FITS-IDI file to a CASA visibility data set

    
    Convert a FITS-IDI file to a CASA visiblity data set.
    If several files are given, they will be concatenated into one MS.

    --------- parameter descriptions ---------------------------------------------

    fitsidifile      Name(s) of input FITS-IDI file(s)
                     Default: none (must be supplied)
                     
                        Examples: 
                        fitsidifile='3C273XC1.IDI'
                        fitsidifile=['3C273XC1.IDI1','3C273XC1.IDI2']
    vis              Name of output visibility file
                     Default: none
                     
                        Example: outputvis='3C273XC1.ms'
    constobsid       If True, give constant obs ID==0 to the data from all
                     input fitsidi files (False = separate obs id for each file)
                     Default: False (new obs id for each input file)
                     Options: False|True
    scanreindexgap_s Min time gap (seconds) between integrations to start a
                     new scan
                     Default: 0. (no reindexing)
                     
                     If > 0., a new scan is started whenever the gap
                     between two integrations is > the given value
                     (seconds) or when a new field starts or when the
                     ARRAY_ID changes.
    specframe        This frame will be used to set the spectral reference
                     frame for all spectral windows in the output MS
                     Default: GEO (geocentric)
                     Options: GEO|TOPO|LSRK|BARY
                     
                     NOTE: if specframe is set to TOPO, the reference
                     location will be taken from the Observatories
                     table in the CASA data repository for the given
                     name of the observatory. You can edit that table
                     and add new rows.

    --------- examples -----------------------------------------------------------

    
    FOR MORE INFORMATION, SEE THE TASK PAGES OF IMPORTFITSIDI IN CASA DOCS:
    https://casa.nrao.edu/casadocs/
    


    """

    _info_group_ = """import/export"""
    _info_desc_ = """Convert a FITS-IDI file to a CASA visibility data set"""

    __schema = {'fitsidifile': {'type': 'cReqPathVec', 'coerce': [_coerce.to_list,_coerce.expand_pathvec]}, 'vis': {'type': 'cStr', 'coerce': _coerce.to_str}, 'constobsid': {'type': 'cBool'}, 'scanreindexgap_s': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'specframe': {'type': 'cStr', 'coerce': _coerce.to_str}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 16 + 4
        output = [ ]
        addon = ''
        first_addon = True
        if len(description) == 0:
            out.write(param_prefix + " #\n")
            return
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __vis_dflt( self, glb ):
        return ''

    def __vis( self, glb ):
        if 'vis' in glb: return glb['vis']
        return ''

    def __constobsid_dflt( self, glb ):
        return False

    def __constobsid( self, glb ):
        if 'constobsid' in glb: return glb['constobsid']
        return False

    def __fitsidifile_dflt( self, glb ):
        return [  ]

    def __fitsidifile( self, glb ):
        if 'fitsidifile' in glb: return glb['fitsidifile']
        return [  ]

    def __scanreindexgap_s_dflt( self, glb ):
        return float(0.)

    def __scanreindexgap_s( self, glb ):
        if 'scanreindexgap_s' in glb: return glb['scanreindexgap_s']
        return float(0.)

    def __specframe_dflt( self, glb ):
        return 'GEO'

    def __specframe( self, glb ):
        if 'specframe' in glb: return glb['specframe']
        return 'GEO'



    #--------- return inp/go default --------------------------------------------------


    #--------- return subparam values -------------------------------------------------


    #--------- subparam inp output ----------------------------------------------------
    def __fitsidifile_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__fitsidifile_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return [  ]
        description = 'Name(s) of input FITS-IDI file(s)'
        value = self.__fitsidifile( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'fitsidifile': value},{'fitsidifile': self.__schema['fitsidifile']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-16.16s = %s%-23s%s' % ('fitsidifile',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __vis_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__vis_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Name of output visibility file'
        value = self.__vis( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'vis': value},{'vis': self.__schema['vis']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-16.16s = %s%-23s%s' % ('vis',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __constobsid_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__constobsid_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'If True, give constant obs ID==0 to the data from all input fitsidi files (False = separate obs id for each file)'
        value = self.__constobsid( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'constobsid': value},{'constobsid': self.__schema['constobsid']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-16.16s = %s%-23s%s' % ('constobsid',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __scanreindexgap_s_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__scanreindexgap_s_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return float(0.)
        description = 'Min time gap (seconds) between integrations to start a new scan'
        value = self.__scanreindexgap_s( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'scanreindexgap_s': value},{'scanreindexgap_s': self.__schema['scanreindexgap_s']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-16.16s = %s%-23s%s' % ('scanreindexgap_s',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __specframe_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__specframe_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'GEO'
        description = 'Spectral reference frame for all spectral windows in the output MS'
        value = self.__specframe( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'specframe': value},{'specframe': self.__schema['specframe']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-16.16s = %s%-23s%s' % ('specframe',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'scanreindexgap_s' in glb: del glb['scanreindexgap_s']
        if 'constobsid' in glb: del glb['constobsid']
        if 'vis' in glb: del glb['vis']
        if 'fitsidifile' in glb: del glb['fitsidifile']
        if 'specframe' in glb: del glb['specframe']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# importfitsidi -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__fitsidifile_inp( )
        self.__vis_inp( )
        self.__constobsid_inp( )
        self.__scanreindexgap_s_inp( )
        self.__specframe_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,savefile=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = savefile
        if filename is None:
            filename = "importfitsidi.last" if os.path.isfile("importfitsidi.last") else "importfitsidi.saved"
        if os.path.isfile(filename):
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file: %s\nsetting defaults instead..." % filename)
            self.set_global_defaults( )

    #--------- tput function ----------------------------------------------------------
    def tput(self,outfile=None):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s

        _postfile = outfile if outfile is not None else os.path.realpath('importfitsidi.last')

        _invocation_parameters = OrderedDict( )
        _invocation_parameters['fitsidifile'] = self.__fitsidifile( self.__globals_( ) )
        _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
        _invocation_parameters['constobsid'] = self.__constobsid( self.__globals_( ) )
        _invocation_parameters['scanreindexgap_s'] = self.__scanreindexgap_s( self.__globals_( ) )
        _invocation_parameters['specframe'] = self.__specframe( self.__globals_( ) )

        try:
            with open(_postfile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-16s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#importfitsidi( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: return False
        return True

    def __call__( self, fitsidifile=None, vis=None, constobsid=None, scanreindexgap_s=None, specframe=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('importfitsidi.pre')
        _postfile = os.path.realpath('importfitsidi.last')
        task_result = None
        _arguments = [fitsidifile,vis,constobsid,scanreindexgap_s,specframe]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if fitsidifile is not None: local_global['fitsidifile'] = fitsidifile
            if vis is not None: local_global['vis'] = vis
            if constobsid is not None: local_global['constobsid'] = constobsid
            if scanreindexgap_s is not None: local_global['scanreindexgap_s'] = scanreindexgap_s
            if specframe is not None: local_global['specframe'] = specframe

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['fitsidifile'] = self.__fitsidifile( local_global )
            _invocation_parameters['vis'] = self.__vis( local_global )
            _invocation_parameters['constobsid'] = self.__constobsid( local_global )
            _invocation_parameters['scanreindexgap_s'] = self.__scanreindexgap_s( local_global )
            _invocation_parameters['specframe'] = self.__specframe( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            

        else:
            # invoke with inp/go semantics
            _invocation_parameters['fitsidifile'] = self.__fitsidifile( self.__globals_( ) )
            _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
            _invocation_parameters['constobsid'] = self.__constobsid( self.__globals_( ) )
            _invocation_parameters['scanreindexgap_s'] = self.__scanreindexgap_s( self.__globals_( ) )
            _invocation_parameters['specframe'] = self.__specframe( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-16s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#importfitsidi( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _logging_state_ = None
            assert _pc.validate(_invocation_parameters,self.__schema), create_error_string(_pc.errors)
            _logging_state_ = _start_log( 'importfitsidi', [ 'fitsidifile=' + repr(_pc.document['fitsidifile']), 'vis=' + repr(_pc.document['vis']), 'constobsid=' + repr(_pc.document['constobsid']), 'scanreindexgap_s=' + repr(_pc.document['scanreindexgap_s']), 'specframe=' + repr(_pc.document['specframe']) ] )
            task_result = _importfitsidi_t( _pc.document['fitsidifile'],_pc.document['vis'],_pc.document['constobsid'],_pc.document['scanreindexgap_s'],_pc.document['specframe'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('importfitsidi')
            casalog.post("Exception Reported: Error in importfitsidi: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            raise #exception is now raised
            #task_result = False
        finally:
            try:
                os.rename(_prefile,_postfile)
            except: pass
            if _logging_state_:
                task_result = _end_log( _logging_state_, 'importfitsidi', task_result )

        #Added if _importfitsidi_t returns False and does not raise an exception.
        if task_result is False:
            raise

        return task_result #Still needed

importfitsidi = _importfitsidi( )

