##################### generated by xml-casa (v2) from importgmrt.xml ################
##################### fd7e58d81ab4ab0e310f3539c4d4f84b ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from casatasks.private.task_importgmrt import importgmrt as _importgmrt_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _importgmrt:
    """
    importgmrt ---- Convert a UVFITS file to a CASA visibility data set

    
    Convert a GRMT FITS file to a CASA visiblity data set.
    Also read GMRT flag file(s) and flag data based on the contents of the
    files.
    

    --------- parameter descriptions ---------------------------------------------

    fitsfile Name of input UV FITS file
             Default: none
             
                Example: fitsimage='3C273XC1.fits'
    flagfile List of files containing flagging information.
             Default: none
             
                Examples:
                flagfile='3c273XC1.flag'
                flagfile=['3c273Cc1_1.flag','3c273Cc2_1.flag',']
    vis      Name of input visibility file
             Default: none
             
                Example: vis='ngc5921.ms'

    --------- examples -----------------------------------------------------------

    
    FOR MORE INFORMATION, SEE THE TASK PAGES OF IMPORTGMRT IN CASA DOCS:
    https://casa.nrao.edu/casadocs/
    


    """

    _info_group_ = """import/export"""
    _info_desc_ = """Convert a UVFITS file to a CASA visibility data set"""

    __schema = {'fitsfile': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'flagfile': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}]}, 'vis': {'type': 'cStr', 'coerce': _coerce.to_str}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 8 + 4
        output = [ ]
        addon = ''
        first_addon = True
        if len(description) == 0:
            out.write(param_prefix + " #\n")
            return
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __fitsfile_dflt( self, glb ):
        return ''

    def __fitsfile( self, glb ):
        if 'fitsfile' in glb: return glb['fitsfile']
        return ''

    def __flagfile_dflt( self, glb ):
        return ''

    def __flagfile( self, glb ):
        if 'flagfile' in glb: return glb['flagfile']
        return ''

    def __vis_dflt( self, glb ):
        return ''

    def __vis( self, glb ):
        if 'vis' in glb: return glb['vis']
        return ''



    #--------- return inp/go default --------------------------------------------------


    #--------- return subparam values -------------------------------------------------


    #--------- subparam inp output ----------------------------------------------------
    def __fitsfile_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__fitsfile_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Name of input UV FITS file'
        value = self.__fitsfile( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'fitsfile': value},{'fitsfile': self.__schema['fitsfile']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-8.8s = %s%-23s%s' % ('fitsfile',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __flagfile_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__flagfile_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'List of files containing flagging information.'
        value = self.__flagfile( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'flagfile': value},{'flagfile': self.__schema['flagfile']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-8.8s = %s%-23s%s' % ('flagfile',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __vis_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__vis_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Name of input visibility file'
        value = self.__vis( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'vis': value},{'vis': self.__schema['vis']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-8.8s = %s%-23s%s' % ('vis',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'fitsfile' in glb: del glb['fitsfile']
        if 'flagfile' in glb: del glb['flagfile']
        if 'vis' in glb: del glb['vis']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# importgmrt -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__fitsfile_inp( )
        self.__flagfile_inp( )
        self.__vis_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,savefile=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = savefile
        if filename is None:
            filename = "importgmrt.last" if os.path.isfile("importgmrt.last") else "importgmrt.saved"
        if os.path.isfile(filename):
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file: %s\nsetting defaults instead..." % filename)
            self.set_global_defaults( )

    #--------- tput function ----------------------------------------------------------
    def tput(self,outfile=None):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s

        _postfile = outfile if outfile is not None else os.path.realpath('importgmrt.last')

        _invocation_parameters = OrderedDict( )
        _invocation_parameters['fitsfile'] = self.__fitsfile( self.__globals_( ) )
        _invocation_parameters['flagfile'] = self.__flagfile( self.__globals_( ) )
        _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )

        try:
            with open(_postfile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-8s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#importgmrt( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: return False
        return True

    def __call__( self, fitsfile=None, flagfile=None, vis=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('importgmrt.pre')
        _postfile = os.path.realpath('importgmrt.last')
        task_result = None
        _arguments = [fitsfile,flagfile,vis]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if fitsfile is not None: local_global['fitsfile'] = fitsfile
            if flagfile is not None: local_global['flagfile'] = flagfile
            if vis is not None: local_global['vis'] = vis

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['fitsfile'] = self.__fitsfile( local_global )
            _invocation_parameters['flagfile'] = self.__flagfile( local_global )
            _invocation_parameters['vis'] = self.__vis( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            

        else:
            # invoke with inp/go semantics
            _invocation_parameters['fitsfile'] = self.__fitsfile( self.__globals_( ) )
            _invocation_parameters['flagfile'] = self.__flagfile( self.__globals_( ) )
            _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-8s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#importgmrt( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _logging_state_ = None
            assert _pc.validate(_invocation_parameters,self.__schema), create_error_string(_pc.errors)
            _logging_state_ = _start_log( 'importgmrt', [ 'fitsfile=' + repr(_pc.document['fitsfile']), 'flagfile=' + repr(_pc.document['flagfile']), 'vis=' + repr(_pc.document['vis']) ] )
            task_result = _importgmrt_t( _pc.document['fitsfile'],_pc.document['flagfile'],_pc.document['vis'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('importgmrt')
            casalog.post("Exception Reported: Error in importgmrt: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            raise #exception is now raised
            #task_result = False
        finally:
            try:
                os.rename(_prefile,_postfile)
            except: pass
            if _logging_state_:
                task_result = _end_log( _logging_state_, 'importgmrt', task_result )

        #Added if _importgmrt_t returns False and does not raise an exception.
        if task_result is False:
            raise

        return task_result #Still needed

importgmrt = _importgmrt( )

