##################### generated by xml-casa (v2) from predictcomp.xml ###############
##################### 72a05d658fe103e7f240bb1a6f564f30 ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from casatasks.private.task_predictcomp import predictcomp as _predictcomp_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _predictcomp:
    """
    predictcomp ---- Make a component list for a known calibrator

    
    Writes a component list named clist to disk and returns a dict of
    {'clist': clist,
    'objname': objname,
    'standard': standard,
    'epoch': epoch,
    'freqs': pl.array of frequencies, in GHz,
    'antennalist': a simdata type configuration file,
    'amps':  pl.array of predicted visibility amplitudes, in Jy,
    'savedfig': False or, if made, the filename of a plot.}
    or False on error.
    

    --------- parameter descriptions ---------------------------------------------

    objname     Object name
    standard    Flux density standard
    epoch       Epoch
    minfreq     Minimum frequency
    maxfreq     Maximum frequency
    nfreqs      Number of frequencies
    prefix      Prefix for the component list directory name.
    antennalist Plot for this configuration
    showplot    Plot S vs \|u\| to the screen?
    savefig     Save a plot of S vs \|u\| to this filename
    symb        A matplotlib plot symbol code
    include0amp Force the amplitude axis to start at 0?
    include0bl  Force the baseline axis to start at 0?
    blunit      unit of the baseline axis
    showbl0flux Print the zero baseline flux ?
    [1;42mRETURNS[1;m        record

    --------- examples -----------------------------------------------------------

    
    
    Writes a component list to disk and returns a dict of
    {'clist': filename of the component list,
    'objname': objname,
    'angdiam': angular diameter in radians (if used in clist),
    'standard': standard,
    'epoch': epoch,
    'freqs': pl.array of frequencies, in GHz,
    'antennalist': pl.array of baseline lengths, in m,
    'amps':  pl.array of predicted visibility amplitudes, in Jy,
    'savedfig': False or, if made, the filename of a plot.}
    or False on error.
    
    objname: An object supported by standard.
    standard: A standard for calculating flux densities, as in setjy.
    Default: 'Butler-JPL-Horizons 2010'
    epoch: The epoch to use for the calculations.   Irrelevant for
    extrasolar standards. (Uses UTC)
    Examples: '2011-12-31/5:34:12', '2011-12-31-5:34:12'
    minfreq: The minimum frequency to use.
    Example: '342.0GHz'
    maxfreq: The maximum frequency to use.
    Default: minfreq
    Example: '346.0GHz'
    Example: '', anything <= 0, or None: use minfreq.
    nfreqs:  The number of frequencies to use.
    Default: 1 if minfreq == maxfreq,
    2 otherwise.
    prefix: The component list will be saved to
    prefix + 'spw0_<objname>_<minfreq><epoch>.cl'
    Default: ''
    Example: "Bands3to7_"
    (which could produce 'Bands3to7_Uranus_spw0_100GHz55877d.cl',
    depending on the other parameters)
    antennalist: 'Observe' and plot the visibility amplitudes for this
    antenna configuration.  The file should be in a format usable
    by simdata.  The search path is:
    .:casa['dirs']['data'] + '/alma/simmos/'
    Default: '' (None, just make clist.)
    Example: 'alma.cycle0.extended.cfg'
    
    Subparameters of antennalist:
    showplot: Whether or not to show a plot of S vs. |u| on screen.
    Subparameter of antennalist.
    Default: Necessarily False if antennalist is not specified.
    True otherwise.
    savefig: Filename for saving a plot of S vs. |u|.
    Subparameter of antennalist.
    Default: ''
    Examples: ''           (do not save the plot)
    'myplot.png' (save to myplot.png)
    symb: One of matplotlib's codes for plot symbols: .:,o^v<>s+xDd234hH|_
    Default: '.'
    include0amp: Force the amplitude axis to start at 0?
    Default: False
    include0bl: Force the baseline axis to start at 0?
    Default: False
    blunit: unit of the baseline axis ('' or 'klambda')
    Default:''=use a unit in the data
    showbl0flux: Print the zero baseline flux?
    Default: False
    
    


    """

    _info_group_ = """imaging, calibration"""
    _info_desc_ = """Make a component list for a known calibrator"""

    __schema = {'objname': {'type': 'cStr', 'coerce': _coerce.to_str}, 'standard': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'Perley-Taylor 95', 'Butler-JPL-Horizons 2010', 'Perley-Butler 2010', 'Perley-Taylor 99', 'Perley-Butler 2013', 'Perley 90', 'Butler-JPL-Horizons 2012', 'Perley-Butler 2017', 'Baars' ]}, 'epoch': {'type': 'cStr', 'coerce': _coerce.to_str}, 'minfreq': {'type': 'cStr', 'coerce': _coerce.to_str}, 'maxfreq': {'type': 'cStr', 'coerce': _coerce.to_str}, 'nfreqs': {'type': 'cInt'}, 'prefix': {'type': 'cPath', 'coerce': _coerce.expand_path}, 'antennalist': {'type': 'cStr', 'coerce': _coerce.to_str}, 'showplot': {'type': 'cBool'}, 'savefig': {'type': 'cStr', 'coerce': _coerce.to_str}, 'symb': {'type': 'cStr', 'coerce': _coerce.to_str}, 'include0amp': {'type': 'cBool'}, 'include0bl': {'type': 'cBool'}, 'blunit': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ '', 'klambda' ]}, 'showbl0flux': {'type': 'cBool'}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 11 + 4
        output = [ ]
        addon = ''
        first_addon = True
        if len(description) == 0:
            out.write(param_prefix + " #\n")
            return
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __nfreqs_dflt( self, glb ):
        return int(2)

    def __nfreqs( self, glb ):
        if 'nfreqs' in glb: return glb['nfreqs']
        return int(2)

    def __objname_dflt( self, glb ):
        return ''

    def __objname( self, glb ):
        if 'objname' in glb: return glb['objname']
        return ''

    def __minfreq_dflt( self, glb ):
        return ''

    def __minfreq( self, glb ):
        if 'minfreq' in glb: return glb['minfreq']
        return ''

    def __prefix_dflt( self, glb ):
        return ''

    def __prefix( self, glb ):
        if 'prefix' in glb: return glb['prefix']
        return ''

    def __epoch_dflt( self, glb ):
        return ''

    def __epoch( self, glb ):
        if 'epoch' in glb: return glb['epoch']
        return ''

    def __standard_dflt( self, glb ):
        return 'Butler-JPL-Horizons 2010'

    def __standard( self, glb ):
        if 'standard' in glb: return glb['standard']
        return 'Butler-JPL-Horizons 2010'

    def __maxfreq_dflt( self, glb ):
        return ''

    def __maxfreq( self, glb ):
        if 'maxfreq' in glb: return glb['maxfreq']
        return ''

    def __antennalist_dflt( self, glb ):
        return ''

    def __antennalist( self, glb ):
        if 'antennalist' in glb: return glb['antennalist']
        return ''

    #--------- return non subparam/when values ---------------------------------------------
    def __showbl0flux( self, glb ):
        if 'showbl0flux' in glb: return glb['showbl0flux']
        return False
    def __showplot( self, glb ):
        if 'showplot' in glb: return glb['showplot']
        return False
    def __symb( self, glb ):
        if 'symb' in glb: return glb['symb']
        return '.'
    def __include0amp( self, glb ):
        if 'include0amp' in glb: return glb['include0amp']
        return False
    def __include0bl( self, glb ):
        if 'include0bl' in glb: return glb['include0bl']
        return False
    def __blunit( self, glb ):
        if 'blunit' in glb: return glb['blunit']
        return ''

    #--------- return inp/go default --------------------------------------------------
    def __blunit_dflt( self, glb ):
        if self.__antennalist( glb ) != "": return ""
        return None
    def __showplot_dflt( self, glb ):
        if self.__antennalist( glb ) != "": return bool(True)
        return None
    def __include0amp_dflt( self, glb ):
        if self.__antennalist( glb ) != "": return bool(False)
        return None
    def __showbl0flux_dflt( self, glb ):
        if self.__antennalist( glb ) != "": return bool(False)
        return None
    def __savefig_dflt( self, glb ):
        if self.__antennalist( glb ) != "": return ""
        return None
    def __symb_dflt( self, glb ):
        if self.__antennalist( glb ) != "": return "."
        return None
    def __include0bl_dflt( self, glb ):
        if self.__antennalist( glb ) != "": return bool(False)
        return None

    #--------- return subparam values -------------------------------------------------
    def __savefig( self, glb ):
        if 'savefig' in glb: return glb['savefig']
        dflt = self.__savefig_dflt( glb )
        if dflt is not None: return dflt
        return ''

    #--------- subparam inp output ----------------------------------------------------
    def __objname_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__objname_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Object name'
        value = self.__objname( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'objname': value},{'objname': self.__schema['objname']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('objname',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __standard_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__standard_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'Butler-JPL-Horizons 2010'
        description = 'Flux density standard'
        value = self.__standard( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'standard': value},{'standard': self.__schema['standard']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('standard',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __epoch_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__epoch_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Epoch'
        value = self.__epoch( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'epoch': value},{'epoch': self.__schema['epoch']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('epoch',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __minfreq_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__minfreq_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Minimum frequency'
        value = self.__minfreq( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'minfreq': value},{'minfreq': self.__schema['minfreq']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('minfreq',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __maxfreq_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__maxfreq_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Maximum frequency'
        value = self.__maxfreq( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'maxfreq': value},{'maxfreq': self.__schema['maxfreq']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('maxfreq',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __nfreqs_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__nfreqs_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return int(2)
        description = 'Number of frequencies'
        value = self.__nfreqs( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'nfreqs': value},{'nfreqs': self.__schema['nfreqs']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('nfreqs',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __prefix_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__prefix_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Prefix for the component list directory name.'
        value = self.__prefix( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'prefix': value},{'prefix': self.__schema['prefix']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('prefix',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __antennalist_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__antennalist_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Plot for this configuration'
        value = self.__antennalist( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'antennalist': value},{'antennalist': self.__schema['antennalist']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-11.11s =\x1B[0m %s%-23s%s' % ('antennalist',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __showplot_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__showplot_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Plot S vs \|u\| to the screen?'
        value = self.__showplot( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'showplot': value},{'showplot': self.__schema['showplot']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('showplot',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __savefig_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__savefig_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__savefig_dflt( self.__globals_( ) ) is not None:
             description = 'Save a plot of S vs \|u\| to this filename'
             value = self.__savefig( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'savefig': value},{'savefig': self.__schema['savefig']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-8.8s =\x1B[0m %s%-23s%s' % ('savefig',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __symb_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__symb_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return '.'
        description = 'A matplotlib plot symbol code'
        value = self.__symb( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'symb': value},{'symb': self.__schema['symb']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('symb',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __include0amp_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__include0amp_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Force the amplitude axis to start at 0?'
        value = self.__include0amp( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'include0amp': value},{'include0amp': self.__schema['include0amp']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('include0amp',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __include0bl_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__include0bl_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Force the baseline axis to start at 0?'
        value = self.__include0bl( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'include0bl': value},{'include0bl': self.__schema['include0bl']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('include0bl',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __blunit_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__blunit_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'unit of the baseline axis'
        value = self.__blunit( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'blunit': value},{'blunit': self.__schema['blunit']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('blunit',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __showbl0flux_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__showbl0flux_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Print the zero baseline flux ?'
        value = self.__showbl0flux( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'showbl0flux': value},{'showbl0flux': self.__schema['showbl0flux']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-11.11s = %s%-23s%s' % ('showbl0flux',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'blunit' in glb: del glb['blunit']
        if 'epoch' in glb: del glb['epoch']
        if 'standard' in glb: del glb['standard']
        if 'prefix' in glb: del glb['prefix']
        if 'showplot' in glb: del glb['showplot']
        if 'include0amp' in glb: del glb['include0amp']
        if 'nfreqs' in glb: del glb['nfreqs']
        if 'showbl0flux' in glb: del glb['showbl0flux']
        if 'objname' in glb: del glb['objname']
        if 'savefig' in glb: del glb['savefig']
        if 'minfreq' in glb: del glb['minfreq']
        if 'symb' in glb: del glb['symb']
        if 'maxfreq' in glb: del glb['maxfreq']
        if 'include0bl' in glb: del glb['include0bl']
        if 'antennalist' in glb: del glb['antennalist']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# predictcomp -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__objname_inp( )
        self.__standard_inp( )
        self.__epoch_inp( )
        self.__minfreq_inp( )
        self.__maxfreq_inp( )
        self.__nfreqs_inp( )
        self.__prefix_inp( )
        self.__antennalist_inp( )
        self.__showplot_inp( )
        self.__savefig_inp( )
        self.__symb_inp( )
        self.__include0amp_inp( )
        self.__include0bl_inp( )
        self.__blunit_inp( )
        self.__showbl0flux_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,savefile=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = savefile
        if filename is None:
            filename = "predictcomp.last" if os.path.isfile("predictcomp.last") else "predictcomp.saved"
        if os.path.isfile(filename):
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file: %s\nsetting defaults instead..." % filename)
            self.set_global_defaults( )

    #--------- tput function ----------------------------------------------------------
    def tput(self,outfile=None):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s

        _postfile = outfile if outfile is not None else os.path.realpath('predictcomp.last')

        _invocation_parameters = OrderedDict( )
        _invocation_parameters['objname'] = self.__objname( self.__globals_( ) )
        _invocation_parameters['standard'] = self.__standard( self.__globals_( ) )
        _invocation_parameters['epoch'] = self.__epoch( self.__globals_( ) )
        _invocation_parameters['minfreq'] = self.__minfreq( self.__globals_( ) )
        _invocation_parameters['maxfreq'] = self.__maxfreq( self.__globals_( ) )
        _invocation_parameters['nfreqs'] = self.__nfreqs( self.__globals_( ) )
        _invocation_parameters['prefix'] = self.__prefix( self.__globals_( ) )
        _invocation_parameters['antennalist'] = self.__antennalist( self.__globals_( ) )
        _invocation_parameters['showplot'] = self.__showplot( self.__globals_( ) )
        _invocation_parameters['savefig'] = self.__savefig( self.__globals_( ) )
        _invocation_parameters['symb'] = self.__symb( self.__globals_( ) )
        _invocation_parameters['include0amp'] = self.__include0amp( self.__globals_( ) )
        _invocation_parameters['include0bl'] = self.__include0bl( self.__globals_( ) )
        _invocation_parameters['blunit'] = self.__blunit( self.__globals_( ) )
        _invocation_parameters['showbl0flux'] = self.__showbl0flux( self.__globals_( ) )

        try:
            with open(_postfile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-11s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#predictcomp( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: return False
        return True

    def __call__( self, objname=None, standard=None, epoch=None, minfreq=None, maxfreq=None, nfreqs=None, prefix=None, antennalist=None, showplot=None, savefig=None, symb=None, include0amp=None, include0bl=None, blunit=None, showbl0flux=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('predictcomp.pre')
        _postfile = os.path.realpath('predictcomp.last')
        task_result = None
        _arguments = [objname,standard,epoch,minfreq,maxfreq,nfreqs,prefix,antennalist,showplot,savefig,symb,include0amp,include0bl,blunit,showbl0flux]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if objname is not None: local_global['objname'] = objname
            if standard is not None: local_global['standard'] = standard
            if epoch is not None: local_global['epoch'] = epoch
            if minfreq is not None: local_global['minfreq'] = minfreq
            if maxfreq is not None: local_global['maxfreq'] = maxfreq
            if nfreqs is not None: local_global['nfreqs'] = nfreqs
            if prefix is not None: local_global['prefix'] = prefix
            if antennalist is not None: local_global['antennalist'] = antennalist
            if showplot is not None: local_global['showplot'] = showplot
            if symb is not None: local_global['symb'] = symb
            if include0amp is not None: local_global['include0amp'] = include0amp
            if include0bl is not None: local_global['include0bl'] = include0bl
            if blunit is not None: local_global['blunit'] = blunit
            if showbl0flux is not None: local_global['showbl0flux'] = showbl0flux

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['objname'] = self.__objname( local_global )
            _invocation_parameters['standard'] = self.__standard( local_global )
            _invocation_parameters['epoch'] = self.__epoch( local_global )
            _invocation_parameters['minfreq'] = self.__minfreq( local_global )
            _invocation_parameters['maxfreq'] = self.__maxfreq( local_global )
            _invocation_parameters['nfreqs'] = self.__nfreqs( local_global )
            _invocation_parameters['prefix'] = self.__prefix( local_global )
            _invocation_parameters['antennalist'] = self.__antennalist( local_global )
            _invocation_parameters['showplot'] = self.__showplot( local_global )
            _invocation_parameters['symb'] = self.__symb( local_global )
            _invocation_parameters['include0amp'] = self.__include0amp( local_global )
            _invocation_parameters['include0bl'] = self.__include0bl( local_global )
            _invocation_parameters['blunit'] = self.__blunit( local_global )
            _invocation_parameters['showbl0flux'] = self.__showbl0flux( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            _invocation_parameters['savefig'] = self.__savefig( _invocation_parameters ) if savefig is None else savefig

        else:
            # invoke with inp/go semantics
            _invocation_parameters['objname'] = self.__objname( self.__globals_( ) )
            _invocation_parameters['standard'] = self.__standard( self.__globals_( ) )
            _invocation_parameters['epoch'] = self.__epoch( self.__globals_( ) )
            _invocation_parameters['minfreq'] = self.__minfreq( self.__globals_( ) )
            _invocation_parameters['maxfreq'] = self.__maxfreq( self.__globals_( ) )
            _invocation_parameters['nfreqs'] = self.__nfreqs( self.__globals_( ) )
            _invocation_parameters['prefix'] = self.__prefix( self.__globals_( ) )
            _invocation_parameters['antennalist'] = self.__antennalist( self.__globals_( ) )
            _invocation_parameters['showplot'] = self.__showplot( self.__globals_( ) )
            _invocation_parameters['savefig'] = self.__savefig( self.__globals_( ) )
            _invocation_parameters['symb'] = self.__symb( self.__globals_( ) )
            _invocation_parameters['include0amp'] = self.__include0amp( self.__globals_( ) )
            _invocation_parameters['include0bl'] = self.__include0bl( self.__globals_( ) )
            _invocation_parameters['blunit'] = self.__blunit( self.__globals_( ) )
            _invocation_parameters['showbl0flux'] = self.__showbl0flux( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-11s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#predictcomp( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _logging_state_ = None
            assert _pc.validate(_invocation_parameters,self.__schema), create_error_string(_pc.errors)
            _logging_state_ = _start_log( 'predictcomp', [ 'objname=' + repr(_pc.document['objname']), 'standard=' + repr(_pc.document['standard']), 'epoch=' + repr(_pc.document['epoch']), 'minfreq=' + repr(_pc.document['minfreq']), 'maxfreq=' + repr(_pc.document['maxfreq']), 'nfreqs=' + repr(_pc.document['nfreqs']), 'prefix=' + repr(_pc.document['prefix']), 'antennalist=' + repr(_pc.document['antennalist']), 'showplot=' + repr(_pc.document['showplot']), 'savefig=' + repr(_pc.document['savefig']), 'symb=' + repr(_pc.document['symb']), 'include0amp=' + repr(_pc.document['include0amp']), 'include0bl=' + repr(_pc.document['include0bl']), 'blunit=' + repr(_pc.document['blunit']), 'showbl0flux=' + repr(_pc.document['showbl0flux']) ] )
            task_result = _predictcomp_t( _pc.document['objname'],_pc.document['standard'],_pc.document['epoch'],_pc.document['minfreq'],_pc.document['maxfreq'],_pc.document['nfreqs'],_pc.document['prefix'],_pc.document['antennalist'],_pc.document['showplot'],_pc.document['savefig'],_pc.document['symb'],_pc.document['include0amp'],_pc.document['include0bl'],_pc.document['blunit'],_pc.document['showbl0flux'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('predictcomp')
            casalog.post("Exception Reported: Error in predictcomp: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            raise #exception is now raised
            #task_result = False
        finally:
            try:
                os.rename(_prefile,_postfile)
            except: pass
            if _logging_state_:
                task_result = _end_log( _logging_state_, 'predictcomp', task_result )

        #Added if _predictcomp_t returns False and does not raise an exception.
        if task_result is False:
            raise

        return task_result #Still needed

predictcomp = _predictcomp( )

