##################### generated by xml-casa (v2) from simobserve.xml ################
##################### 0e3aaa53282e4e44cb7a3dfe57d8f593 ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from casatasks.private.task_simobserve import simobserve as _simobserve_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _simobserve:
    """
    simobserve ---- visibility simulation task

    
    This task simulates interferometric or total power measurment sets. It
    is currently optimized for JVLA and ALMA, although many observatories
    are included, and adding your own is simply a matter of providing an
    antenna location file (see below).
    
    simobserve is meant to work in conjunction with the simanalyze
    task. Calling simobserve one more times will produce simulated
    measurement set(s), which are then gridded, inverted and deconvolved
    into output simulated images using simanalyze.
    
    ALMA users are encouraged to use the simalma task, which provides
    additional information on the multiple simobserve and simanalyze calls
    required to simulate an ALMA observation which may consist of 12m
    interferometric, 7m interferometric, and 12m total power data.
    
    More information and examples are availible in the CASA Guides (http://casaguides.nrao.edu) under Simulations.
    Please contact the CASA Team with any questions (see
    https://casa.nrao.edu/index_contact.shtml)

    --------- parameter descriptions ---------------------------------------------

    project         root prefix for output file names
    skymodel        Model image to observe
                    
                    * simobserve uses a CASA or fits image. If you
                      merely have a grid of numbers, you will need to
                      write them out as fits or write a CASA script to
                      read them in and use the ia tool to create an
                      image and insert the data.
                    
                    * simobserve does NOT require a coordinate system
                      in the header. If the coordinate information is
                      incomplete, missing, or you would like to
                      override it, set the appropriate "in"
                      parameters. NOTE that setting those parameters
                      simply changes the header values, ignoring any
                      values already in the image. No regridding is
                      performed. 
                    
                    * You can also manipulate an image header manually
                      with the "imhead" task. 
                    
                    * If you have a proper Coordinate System,
                      simobserve will do its best to generate
                      visibilities from that.
    inbright        Peak brightness to scale the image to, in Jy/pixel
                    Subparameter of skymodel
                    Default: '' (i.e., unchanged)
                    
                       Example: inbright='1.2Jy/pixel'
                    
                    Note: "unchanged" will take the numerical values
                    in your image and assume they are in Jy/pixel,
                    even if it says some other unit in the header.
    indirection     Central direction to place the sky model image
                    Subparameter of skymodel
                    Default: '' (use whatever is in the image
                    already)
                    
                       Example: indirection='J2000 19h00m00
                       -40d00m00'
    incell          set new cell/pixel size
                    Subparameter of skymodel
                    Default: '' (use whatever is in the image
                    already)
                    
                       Example: incell='0.1arcsec'
    incenter        Frequency to use for the center channel (or only channel,
                    if the skymodel is 2D).
                    Subparameter of skymodel
                    Default: '' (use whatever is in the image
                    already)
                    
                    		     This will also become the default rest frequency, e.g. when imaging with tclean.
                    
                       Example: incenter='89GHz'
    inwidth         Set new channel width 
                      Subparameter of skymodel
                      Default: '' (use whatever is in the image
                      already)
                      
                      Should be a string representing a quantity with
                      units e.g. inwidth='10MHz'
                    
                      NOTES: 
                    * Only works reliably with frequencies, not
                      velocities 
                    * It is not possible to change the number of
                      spectral planes of the sky model, only to relabel
                      them with different frequencies That kind of
                      regridding can be accomplished with the CASA
                      toolkit.
    complist        Component list model of the sky, added to or instead of skymodel.
    compwidth       Bandwidth of components
                    Subparameter of complist
                    
                    If simulating from components only, this defines
                    the bandwidth of the MS and output images
                    
                       Example: compwidth='8GHz'
    comp_nchan      Channelization of components
                    Subparameter of complist
                    
                    If simulating from components only, this defines
                    the number of channels of the MeasurementSet
                    
                       Example: comp_nchan=256
    setpointings    If true, calculate a map of pointings and write ptgfile. If false, read pointings from ptgfile.
                    Default: True
                    
                    If graphics are on, display the pointings shown
                    on the model image
    ptgfile         A text file specifying directions
                    Subparameter of setpointings=False
                    
                    The text file should have the following format,
                    with optional integration times:
                    Epoch     RA          DEC      TIME(optional)
                    J2000 23h59m28.10 -019d52m12.35 10.0
                    
                    If the time column is not present in the file, it
                    will use "integration" for all pointings.
                    
                    NOTE: at this time the file should contain only
                    science pointings: simobserve will observe these,
                    then optionally the calibrator, then the list of
                    science pointings again, etc, until totaltime is
                    used up.
    integration     Time interval for each integration
                    Subparameter of setpointings=False
                    
                       Example: integration='10s'
                    
                    NOTE: to simulate a "scan" longer than one
                    integration, use  setpointings to generate a
                    pointing file, and then edit the file to increase
                    the time at each point to be larger than the
                    parameter integration time.
    direction       Mosaic center direction.
                      Subparameter of setpointings=True
                    
                         Example: "J2000 19h00m00 -40d00m00" or "" to
                         center on model
                    
                      If unset, will use the center of the skymodel
                      image.
                    * can optionally be a list of pointings, otherwise
                    * simobserve will cover a region of size mapsize
                      according to maptype
    mapsize         Angular size of of mosaic map to simulate.
                    Subparameter of setpointings=True
                    
                    Set to "" to cover model
    maptype         How to calculate the pointings for the mosaic
                    observation?
                    Subparameter of setpointings=True
                    Options: hexagonal, square (raster), ALMA, etc
                    
                    "ALMA" for the same hex algorithm as the ALMA
                    Cycle 1 OT or "ALMA2012" for the algorithm used
                    in the Cycle 0 OT
    pointingspacing Spacing in between pointings. 
                    Subparameter of setpointings=True
                    
                       Examples: 
                       pointingspacing="0.25PB" 
                       pointingspacing="" for ALMA default
                       INT=lambda/D/sqrt(3), SD=lambda/D/3
    caldirection    pt source calibrator [experimental]
    calflux         pt source calibrator flux [experimental]
    obsmode         Observation mode to simulate
                      Options: int(interferometer)|sd(singledish)|""(none)
                    
                      Observation mode to calculate visibilities from a
                      skymodel image (which may have been modified
                      above), an optional component list, and a
                      pointing file (which also may have been generated
                      above).
                    
                      This parameter takes two possible values:
                      - interferometer (or int)
                      - singledish (or sd)
                    * If graphics are on, this observe step will
                      display the array (similar to plotants), the uv
                      coverage, the synthesized (dirty) beam, and
                      ephemeris information 
                    * If simulating from a component list, you should
                      specify "compwidth", the desired bandwidth; and 
                    		     specify "comp_nchan", the desired channelization
                    		     if more than one output channel is desired
    refdate         Date of simulated observation
                    Subparameter of obsmode='int|sd'
                    Not critical unless concatting simulations
                    
                       Example: refdate="2014/05/21"
    hourangle       Hour angle of observation center.
                    Subparameter of obsmode='int|sd'
                    
                        Examples:
                        hourangle="-3:00:00", "5h", or "transit"
    totaltime       Total time of observation or number of repetitions
                    Subparameter of obsmode='int|sd'
                    
                        Example:
                        totaltime='7200s'
                        If a number without units, interpreted as the
                        number of times to repeat the mosaic.
    antennalist     Text file containing antenna positions.
                      Subparameter of obsmode='int|""'
                    
                      Each row has x y z coordinates and antenna diameter
                    		     with optional station name and antenna name.
                    		     Header lines are required to specify:
                      # observatory=ALMA
                      # coordsys=UTM
                    
                    		     If the Universal Transverse Mercator projection is
                    		     specified, then other keywords are required:
                      # datum=WGS84
                      # zone=19
                    		     # hemisphere=S
                    
                    		     If the observatory keyword is not defined, then the
                    		     COFA keyword should be, using a coordinate pair:
                    		     #COFA=-67.75,-23.02
                    
                    * Standard array configurations are found in your
                    		     CASA data repository,
                    * If "", simobserve will not not produce an
                      interferometric MS 
                    * A string of the form "alma;0.5arcsec" will be
                    parsed into a full 12m ALMA configuration.
    sdantlist       single dish antenna position file
                    Subparameter of obsmode='sd|""'
    sdant           Index of the antenna in the list to use for total power.  
                    Subparameter of obsmode='sd|""'
                    Default: first antenna on the list.
    outframe        spectral frame of MS to create
                    Subparameter of obsmode='sd|""'
    thermalnoise    add thermal noise.
                                        Options: tsys-atm, tsys-manual, ""
                    
                                        This parameter accepts two settings:
                                        - tsys-atm: J. Pardo's ATM library will be used
                                        to construct an atmospheric profile for the ALMA
                                        site: altitude 5000m, ground pressure 650mbar,
                                        relhum=20%, a water layer of user_pwv at altitude
                                        of 2km, the sky brightness temperature returned
                                        by ATM, and internally tabulated receiver
                                        temperatures.
                                        - tsys-manual: instead of using the ATM model,
                                        specify the zenith  sky brightness and opacity
                                        manually.  Noise is added and then the visibility
                                        flux scale is referenced above the atmosphere.
                    
                                        If left unset (empty string) no thermalnoise
                                        corruption is performed.
                    
                                        In either mode, noise is calculated using an
                                        antenna spillover efficiency of 0.96, taper of
                                        0.86, surface accuracy of 25 and 300 microns for
                                        ALMA and EVLA respectively (using the Ruze
                                        formula for surface efficiency), correlator
                                        efficiencies of 0.95 and 0.91 for ALMA and EVLA,
                                        receiver temperatures 
                                        for ALMA of 17, 30, 37, 51, 65,
                                        83,147,196,175,230 K interpolated between 35,
                                        75,110,145,185,230,345,409,675,867 GHz, 
                                        for EVLA of 500, 70,  60,  55,  100, 130, 350 K
                                        interpolated between
                                        0.33,1.47,4.89,8.44,22.5,33.5,43.3 GHz, 
                                        for SMA of 67,  116, 134, 500 K interpolated
                                        between 212.,310.,383.,660. GHz.
                    
                                        Note: These are only approximate numbers and do
                                        not take into account performance at edges of
                                        receiver bands, neither are they guaranteed to
                                        reflect the most recent measurements.  Caveat
                                        emptor. Use the sm tool to add noise if you want
                                        more precise control, and use the ALMA exposure
                                        time calculator for sensitivity numbers in
                                        proposals.
    user_pwv        Precipitable water vapor if constructing an atmospheric
                    model (in mm)
                    Subparameter of thermalnoise='tsys-atm'
    t_ground        Ground/spillover temperature in K
                    Subparameter of
                    thermalnoise='tsys-atm|tsys-manual'
    t_sky           Atmospheric temperature in K
                    Subparameter of thermalnoise='tsys-manual'
    tau0            Zenith opacity at observing frequency
                    Subparameter of thermalnoise='tsys-manual'
                    
                    See the Simulations CASA Guide (https://casaguides.nrao.edu) on 'Corrupting Simulated Data' for more information on noise, in particular how to add a phase screen using the toolkit
    seed            Random number seed
                    Subparameter of
                    thermalnoise='tsys-atm|tsys-manual'
    leakage         add cross polarization corruption of this fractional
                    magnitude (interferometer only)
    graphics        View plots on the screen, saved to file, both, or neither
                    Options: screen|file|both|none
    verbose         Print extra information to the logger and terminal
                    Default: False
                    Options: True|False
    overwrite       Overwrite files starting with $project
                    Default: False
                    Options: True|False
    [1;42mRETURNS[1;m            bool

    --------- examples -----------------------------------------------------------

    
    
    
    For more information, see the task pages of simobserve in CASA Docs:
    
    https://casa.nrao.edu/casadocs/


    """

    _info_group_ = """simulation"""
    _info_desc_ = """visibility simulation task"""

    __schema = {'project': {'type': 'cStr', 'coerce': _coerce.to_str}, 'skymodel': {'type': 'cStr', 'coerce': _coerce.to_str}, 'inbright': {'type': 'cStr', 'coerce': _coerce.to_str}, 'indirection': {'type': 'cStr', 'coerce': _coerce.to_str}, 'incell': {'type': 'cStr', 'coerce': _coerce.to_str}, 'incenter': {'type': 'cStr', 'coerce': _coerce.to_str}, 'inwidth': {'type': 'cStr', 'coerce': _coerce.to_str}, 'complist': {'type': 'cStr', 'coerce': _coerce.to_str}, 'compwidth': {'type': 'cStr', 'coerce': _coerce.to_str}, 'comp_nchan': {'type': 'cInt'}, 'setpointings': {'type': 'cBool'}, 'ptgfile': {'type': 'cStr', 'coerce': _coerce.to_str}, 'integration': {'type': 'cStr', 'coerce': _coerce.to_str}, 'direction': {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}, 'mapsize': {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}, 'maptype': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'alma', 'square', 'hexagonal', 'ALMA-OT', 'ALMA', 'ALMA2012', 'hex' ]}, 'pointingspacing': {'type': 'cStr', 'coerce': _coerce.to_str}, 'caldirection': {'type': 'cStr', 'coerce': _coerce.to_str}, 'calflux': {'type': 'cStr', 'coerce': _coerce.to_str}, 'obsmode': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ '', 'int', 'sd' ]}, 'refdate': {'type': 'cStr', 'coerce': _coerce.to_str}, 'hourangle': {'type': 'cStr', 'coerce': _coerce.to_str}, 'totaltime': {'type': 'cStr', 'coerce': _coerce.to_str}, 'antennalist': {'type': 'cStr', 'coerce': _coerce.to_str}, 'sdantlist': {'type': 'cStr', 'coerce': _coerce.to_str}, 'sdant': {'type': 'cInt'}, 'outframe': {'type': 'cStr', 'coerce': _coerce.to_str}, 'thermalnoise': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ '', 'tsys-atm', 'tsys-manual' ]}, 'user_pwv': {'type': 'cFloat', 'coerce': _coerce.to_float, 'min': 0}, 't_ground': {'type': 'cFloat', 'coerce': _coerce.to_float, 'min': 0}, 't_sky': {'type': 'cFloat', 'coerce': _coerce.to_float, 'min': 0}, 'tau0': {'type': 'cFloat', 'coerce': _coerce.to_float, 'min': 0}, 'seed': {'type': 'cInt'}, 'leakage': {'type': 'cFloat', 'coerce': _coerce.to_float, 'min': 0}, 'graphics': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'none', 'file', 'both', 'screen', '' ]}, 'verbose': {'type': 'cBool'}, 'overwrite': {'type': 'cBool'}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 18 + 4
        output = [ ]
        addon = ''
        first_addon = True
        if len(description) == 0:
            out.write(param_prefix + " #\n")
            return
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __skymodel_dflt( self, glb ):
        return ''

    def __skymodel( self, glb ):
        if 'skymodel' in glb: return glb['skymodel']
        return ''

    def __setpointings_dflt( self, glb ):
        return True

    def __setpointings( self, glb ):
        if 'setpointings' in glb: return glb['setpointings']
        return True

    def __overwrite_dflt( self, glb ):
        return True

    def __overwrite( self, glb ):
        if 'overwrite' in glb: return glb['overwrite']
        return True

    def __obsmode_dflt( self, glb ):
        return 'int'

    def __obsmode( self, glb ):
        if 'obsmode' in glb: return glb['obsmode']
        return 'int'

    def __thermalnoise_dflt( self, glb ):
        return 'tsys-atm'

    def __thermalnoise( self, glb ):
        if 'thermalnoise' in glb: return glb['thermalnoise']
        return 'tsys-atm'

    def __project_dflt( self, glb ):
        return 'sim'

    def __project( self, glb ):
        if 'project' in glb: return glb['project']
        return 'sim'

    def __leakage_dflt( self, glb ):
        return float(0.0)

    def __leakage( self, glb ):
        if 'leakage' in glb: return glb['leakage']
        return float(0.0)

    def __graphics_dflt( self, glb ):
        return 'both'

    def __graphics( self, glb ):
        if 'graphics' in glb: return glb['graphics']
        return 'both'

    def __outframe_dflt( self, glb ):
        return 'LSRK'

    def __outframe( self, glb ):
        if 'outframe' in glb: return glb['outframe']
        return 'LSRK'

    def __complist_dflt( self, glb ):
        return ''

    def __complist( self, glb ):
        if 'complist' in glb: return glb['complist']
        return ''

    def __verbose_dflt( self, glb ):
        return False

    def __verbose( self, glb ):
        if 'verbose' in glb: return glb['verbose']
        return False



    #--------- return inp/go default --------------------------------------------------
    def __sdant_dflt( self, glb ):
        if self.__obsmode( glb ) == "sd": return int(0)
        if self.__obsmode( glb ) == "": return int(0)
        return None
    def __hourangle_dflt( self, glb ):
        if self.__obsmode( glb ) == "int": return "transit"
        if self.__obsmode( glb ) == "sd": return "transit"
        return None
    def __totaltime_dflt( self, glb ):
        if self.__obsmode( glb ) == "int": return "7200s"
        if self.__obsmode( glb ) == "sd": return "7200s"
        return None
    def __pointingspacing_dflt( self, glb ):
        if self.__setpointings( glb ) == bool(True): return ""
        return None
    def __refdate_dflt( self, glb ):
        if self.__obsmode( glb ) == "int": return "2014/05/21"
        if self.__obsmode( glb ) == "sd": return "2014/05/21"
        return None
    def __ptgfile_dflt( self, glb ):
        if self.__setpointings( glb ) == bool(False): return "$project.ptg.txt"
        return None
    def __incenter_dflt( self, glb ):
        if self.__skymodel( glb ) != "": return ""
        return None
    def __seed_dflt( self, glb ):
        if self.__thermalnoise( glb ) == "tsys-atm": return int(11111)
        if self.__thermalnoise( glb ) == "tsys-manual": return int(11111)
        return None
    def __integration_dflt( self, glb ):
        if self.__setpointings( glb ) == bool(True): return "10s"
        if self.__setpointings( glb ) == bool(False): return "10s"
        return None
    def __calflux_dflt( self, glb ):
        if self.__obsmode( glb ) == "int": return "1Jy"
        return None
    def __user_pwv_dflt( self, glb ):
        if self.__thermalnoise( glb ) == "tsys-atm": return float(0.5)
        return None
    def __direction_dflt( self, glb ):
        if self.__setpointings( glb ) == bool(True): return []
        return None
    def __compwidth_dflt( self, glb ):
        if self.__complist( glb ) != "": return "8GHz"
        return None
    def __maptype_dflt( self, glb ):
        if self.__setpointings( glb ) == bool(True): return "ALMA"
        return None
    def __incell_dflt( self, glb ):
        if self.__skymodel( glb ) != "": return ""
        return None
    def __indirection_dflt( self, glb ):
        if self.__skymodel( glb ) != "": return ""
        return None
    def __mapsize_dflt( self, glb ):
        if self.__setpointings( glb ) == bool(True): return ["",""]
        return None
    def __inbright_dflt( self, glb ):
        if self.__skymodel( glb ) != "": return ""
        return None
    def __inwidth_dflt( self, glb ):
        if self.__skymodel( glb ) != "": return ""
        return None
    def __antennalist_dflt( self, glb ):
        if self.__obsmode( glb ) == "int": return "alma.out10.cfg"
        if self.__obsmode( glb ) == "": return ""
        return None
    def __sdantlist_dflt( self, glb ):
        if self.__obsmode( glb ) == "sd": return "aca.tp.cfg"
        if self.__obsmode( glb ) == "": return ""
        return None
    def __caldirection_dflt( self, glb ):
        if self.__obsmode( glb ) == "int": return ""
        return None
    def __t_ground_dflt( self, glb ):
        if self.__thermalnoise( glb ) == "tsys-atm": return float(269.)
        if self.__thermalnoise( glb ) == "tsys-manual": return float(269.)
        return None
    def __tau0_dflt( self, glb ):
        if self.__thermalnoise( glb ) == "tsys-manual": return float(0.1)
        return None
    def __t_sky_dflt( self, glb ):
        if self.__thermalnoise( glb ) == "tsys-manual": return float(263.)
        return None
    def __comp_nchan_dflt( self, glb ):
        if self.__complist( glb ) != "": return int(1)
        return None

    #--------- return subparam values -------------------------------------------------
    def __inbright( self, glb ):
        if 'inbright' in glb: return glb['inbright']
        dflt = self.__inbright_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __indirection( self, glb ):
        if 'indirection' in glb: return glb['indirection']
        dflt = self.__indirection_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __incell( self, glb ):
        if 'incell' in glb: return glb['incell']
        dflt = self.__incell_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __incenter( self, glb ):
        if 'incenter' in glb: return glb['incenter']
        dflt = self.__incenter_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __inwidth( self, glb ):
        if 'inwidth' in glb: return glb['inwidth']
        dflt = self.__inwidth_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __compwidth( self, glb ):
        if 'compwidth' in glb: return glb['compwidth']
        dflt = self.__compwidth_dflt( glb )
        if dflt is not None: return dflt
        return '"8GHz"'
    def __comp_nchan( self, glb ):
        if 'comp_nchan' in glb: return glb['comp_nchan']
        dflt = self.__comp_nchan_dflt( glb )
        if dflt is not None: return dflt
        return int(1)
    def __ptgfile( self, glb ):
        if 'ptgfile' in glb: return glb['ptgfile']
        dflt = self.__ptgfile_dflt( glb )
        if dflt is not None: return dflt
        return '$project.ptg.txt'
    def __integration( self, glb ):
        if 'integration' in glb: return glb['integration']
        dflt = self.__integration_dflt( glb )
        if dflt is not None: return dflt
        return '10s'
    def __direction( self, glb ):
        if 'direction' in glb: return glb['direction']
        dflt = self.__direction_dflt( glb )
        if dflt is not None: return dflt
        return [  ]
    def __mapsize( self, glb ):
        if 'mapsize' in glb: return glb['mapsize']
        dflt = self.__mapsize_dflt( glb )
        if dflt is not None: return dflt
        return [ '','' ]
    def __maptype( self, glb ):
        if 'maptype' in glb: return glb['maptype']
        dflt = self.__maptype_dflt( glb )
        if dflt is not None: return dflt
        return 'hexagonal'
    def __pointingspacing( self, glb ):
        if 'pointingspacing' in glb: return glb['pointingspacing']
        dflt = self.__pointingspacing_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __caldirection( self, glb ):
        if 'caldirection' in glb: return glb['caldirection']
        dflt = self.__caldirection_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __calflux( self, glb ):
        if 'calflux' in glb: return glb['calflux']
        dflt = self.__calflux_dflt( glb )
        if dflt is not None: return dflt
        return '1Jy'
    def __refdate( self, glb ):
        if 'refdate' in glb: return glb['refdate']
        dflt = self.__refdate_dflt( glb )
        if dflt is not None: return dflt
        return '2014/01/01'
    def __hourangle( self, glb ):
        if 'hourangle' in glb: return glb['hourangle']
        dflt = self.__hourangle_dflt( glb )
        if dflt is not None: return dflt
        return 'transit'
    def __totaltime( self, glb ):
        if 'totaltime' in glb: return glb['totaltime']
        dflt = self.__totaltime_dflt( glb )
        if dflt is not None: return dflt
        return '7200s'
    def __antennalist( self, glb ):
        if 'antennalist' in glb: return glb['antennalist']
        dflt = self.__antennalist_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __sdantlist( self, glb ):
        if 'sdantlist' in glb: return glb['sdantlist']
        dflt = self.__sdantlist_dflt( glb )
        if dflt is not None: return dflt
        return 'aca.tp.cfg'
    def __sdant( self, glb ):
        if 'sdant' in glb: return glb['sdant']
        dflt = self.__sdant_dflt( glb )
        if dflt is not None: return dflt
        return int(0)
    def __user_pwv( self, glb ):
        if 'user_pwv' in glb: return glb['user_pwv']
        dflt = self.__user_pwv_dflt( glb )
        if dflt is not None: return dflt
        return float(0.5)
    def __t_ground( self, glb ):
        if 't_ground' in glb: return glb['t_ground']
        dflt = self.__t_ground_dflt( glb )
        if dflt is not None: return dflt
        return float(270.)
    def __t_sky( self, glb ):
        if 't_sky' in glb: return glb['t_sky']
        dflt = self.__t_sky_dflt( glb )
        if dflt is not None: return dflt
        return float(260.)
    def __tau0( self, glb ):
        if 'tau0' in glb: return glb['tau0']
        dflt = self.__tau0_dflt( glb )
        if dflt is not None: return dflt
        return float(0.1)
    def __seed( self, glb ):
        if 'seed' in glb: return glb['seed']
        dflt = self.__seed_dflt( glb )
        if dflt is not None: return dflt
        return int(11111)

    #--------- subparam inp output ----------------------------------------------------
    def __project_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__project_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'sim'
        description = 'Root prefix for output file names'
        value = self.__project( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'project': value},{'project': self.__schema['project']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-18.18s = %s%-23s%s' % ('project',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __skymodel_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__skymodel_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'model image to observe'
        value = self.__skymodel( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'skymodel': value},{'skymodel': self.__schema['skymodel']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-18.18s =\x1B[0m %s%-23s%s' % ('skymodel',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __inbright_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__inbright_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__inbright_dflt( self.__globals_( ) ) is not None:
             description = 'Peak brightness to scale the image to in Jy/pixel'
             value = self.__inbright( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'inbright': value},{'inbright': self.__schema['inbright']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('inbright',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __indirection_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__indirection_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__indirection_dflt( self.__globals_( ) ) is not None:
             description = 'Set new direction, e.g. J2000 19h00m00 -40d00m00'
             value = self.__indirection( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'indirection': value},{'indirection': self.__schema['indirection']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('indirection',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __incell_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__incell_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__incell_dflt( self.__globals_( ) ) is not None:
             description = 'Set new cell/pixel size, e.g. 0.1arcsec'
             value = self.__incell( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'incell': value},{'incell': self.__schema['incell']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('incell',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __incenter_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__incenter_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__incenter_dflt( self.__globals_( ) ) is not None:
             description = 'Set new frequency of center channel e.g. 89GHz (required even for 2D model)'
             value = self.__incenter( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'incenter': value},{'incenter': self.__schema['incenter']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('incenter',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __inwidth_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__inwidth_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__inwidth_dflt( self.__globals_( ) ) is not None:
             description = 'Set new channel width, e.g. "10MHz" (required even for 2D model)'
             value = self.__inwidth( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'inwidth': value},{'inwidth': self.__schema['inwidth']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('inwidth',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __complist_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__complist_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Componentlist to observe'
        value = self.__complist( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'complist': value},{'complist': self.__schema['complist']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-18.18s =\x1B[0m %s%-23s%s' % ('complist',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __compwidth_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__compwidth_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return '"8GHz"'
        if self.__compwidth_dflt( self.__globals_( ) ) is not None:
             description = 'Bandwidth of components'
             value = self.__compwidth( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'compwidth': value},{'compwidth': self.__schema['compwidth']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('compwidth',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __comp_nchan_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__comp_nchan_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return int(1)
        if self.__comp_nchan_dflt( self.__globals_( ) ) is not None:
             description = 'Channelization of components'
             value = self.__comp_nchan( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'comp_nchan': value},{'comp_nchan': self.__schema['comp_nchan']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('comp_nchan',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __setpointings_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__setpointings_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return True
        description = 'Calculate a map of pointings?'
        value = self.__setpointings( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'setpointings': value},{'setpointings': self.__schema['setpointings']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-18.18s =\x1B[0m %s%-23s%s' % ('setpointings',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __ptgfile_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__ptgfile_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return '$project.ptg.txt'
        if self.__ptgfile_dflt( self.__globals_( ) ) is not None:
             description = 'List of pointing positions'
             value = self.__ptgfile( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'ptgfile': value},{'ptgfile': self.__schema['ptgfile']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('ptgfile',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __integration_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__integration_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return '10s'
        if self.__integration_dflt( self.__globals_( ) ) is not None:
             description = 'Integration (sampling) time'
             value = self.__integration( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'integration': value},{'integration': self.__schema['integration']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('integration',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __direction_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__direction_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return [  ]
        if self.__direction_dflt( self.__globals_( ) ) is not None:
             description = 'Mosaic center direction, e.g J2000 19h00m00 -40d00m00'
             value = self.__direction( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'direction': value},{'direction': self.__schema['direction']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('direction',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __mapsize_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__mapsize_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return [ '','' ]
        if self.__mapsize_dflt( self.__globals_( ) ) is not None:
             description = 'Angular size of mosaic map to simulate.'
             value = self.__mapsize( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'mapsize': value},{'mapsize': self.__schema['mapsize']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('mapsize',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __maptype_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__maptype_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'hexagonal'
        if self.__maptype_dflt( self.__globals_( ) ) is not None:
             description = 'how to calculate the pointings for the mosaic observation: hexagonal, square (raster), ALMA, etc.'
             value = self.__maptype( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'maptype': value},{'maptype': self.__schema['maptype']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('maptype',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __pointingspacing_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__pointingspacing_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__pointingspacing_dflt( self.__globals_( ) ) is not None:
             description = 'Spacing in between pointings e.g. 0.25PB. ALMA default: INT=lambda/D/sqrt(3), SD=lambda/D/3'
             value = self.__pointingspacing( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'pointingspacing': value},{'pointingspacing': self.__schema['pointingspacing']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('pointingspacing',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __caldirection_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__caldirection_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__caldirection_dflt( self.__globals_( ) ) is not None:
             description = 'pt source calibrator [experimental]'
             value = self.__caldirection( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'caldirection': value},{'caldirection': self.__schema['caldirection']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('caldirection',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __calflux_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__calflux_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return '1Jy'
        if self.__calflux_dflt( self.__globals_( ) ) is not None:
             description = 'pt source calibrator flux [experimental]'
             value = self.__calflux( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'calflux': value},{'calflux': self.__schema['calflux']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('calflux',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __obsmode_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__obsmode_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'int'
        description = 'Observation mode to simulate [int(interferometer)|sd(singledish)|(none)]'
        value = self.__obsmode( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'obsmode': value},{'obsmode': self.__schema['obsmode']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-18.18s =\x1B[0m %s%-23s%s' % ('obsmode',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __refdate_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__refdate_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return '2014/01/01'
        if self.__refdate_dflt( self.__globals_( ) ) is not None:
             description = 'Date of observation. Not critical unless concatting simulations'
             value = self.__refdate( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'refdate': value},{'refdate': self.__schema['refdate']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('refdate',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __hourangle_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__hourangle_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'transit'
        if self.__hourangle_dflt( self.__globals_( ) ) is not None:
             description = 'Hour angle of observation center, e.g. -3:00:00, 5h'
             value = self.__hourangle( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'hourangle': value},{'hourangle': self.__schema['hourangle']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('hourangle',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __totaltime_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__totaltime_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return '7200s'
        if self.__totaltime_dflt( self.__globals_( ) ) is not None:
             description = 'Total time of observation or number of repetitions'
             value = self.__totaltime( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'totaltime': value},{'totaltime': self.__schema['totaltime']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('totaltime',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __antennalist_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__antennalist_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        if self.__antennalist_dflt( self.__globals_( ) ) is not None:
             description = 'Interferometer antenna position file'
             value = self.__antennalist( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'antennalist': value},{'antennalist': self.__schema['antennalist']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('antennalist',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __sdantlist_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__sdantlist_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'aca.tp.cfg'
        if self.__sdantlist_dflt( self.__globals_( ) ) is not None:
             description = 'Single dish antenna position file'
             value = self.__sdantlist( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'sdantlist': value},{'sdantlist': self.__schema['sdantlist']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('sdantlist',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __sdant_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__sdant_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return int(0)
        if self.__sdant_dflt( self.__globals_( ) ) is not None:
             description = 'Single dish antenna index in file'
             value = self.__sdant( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'sdant': value},{'sdant': self.__schema['sdant']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('sdant',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __outframe_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__outframe_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'LSRK'
        description = 'Spectral frame of MS to create'
        value = self.__outframe( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'outframe': value},{'outframe': self.__schema['outframe']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-18.18s = %s%-23s%s' % ('outframe',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __thermalnoise_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__thermalnoise_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'tsys-atm'
        description = 'add thermal noise: [tsys-atm|tsys-manual|(none)]'
        value = self.__thermalnoise( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'thermalnoise': value},{'thermalnoise': self.__schema['thermalnoise']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-18.18s =\x1B[0m %s%-23s%s' % ('thermalnoise',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __user_pwv_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__user_pwv_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return float(0.5)
        if self.__user_pwv_dflt( self.__globals_( ) ) is not None:
             description = 'Precipitable Water Vapor in mm'
             value = self.__user_pwv( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'user_pwv': value},{'user_pwv': self.__schema['user_pwv']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('user_pwv',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __t_ground_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__t_ground_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return float(270.)
        if self.__t_ground_dflt( self.__globals_( ) ) is not None:
             description = 'Ground/spillover ambient temperature in K'
             value = self.__t_ground( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'t_ground': value},{'t_ground': self.__schema['t_ground']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('t_ground',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __t_sky_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__t_sky_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return float(260.)
        if self.__t_sky_dflt( self.__globals_( ) ) is not None:
             description = 'Atmospheric temperatur in K'
             value = self.__t_sky( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'t_sky': value},{'t_sky': self.__schema['t_sky']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('t_sky',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __tau0_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__tau0_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return float(0.1)
        if self.__tau0_dflt( self.__globals_( ) ) is not None:
             description = 'Zenith opacity'
             value = self.__tau0( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'tau0': value},{'tau0': self.__schema['tau0']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('tau0',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __seed_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__seed_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return int(11111)
        if self.__seed_dflt( self.__globals_( ) ) is not None:
             description = 'Random number seed'
             value = self.__seed( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'seed': value},{'seed': self.__schema['seed']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-15.15s =\x1B[0m %s%-23s%s' % ('seed',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __leakage_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__leakage_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return float(0.0)
        description = 'Cross polarization (interferometer only)'
        value = self.__leakage( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'leakage': value},{'leakage': self.__schema['leakage']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-18.18s = %s%-23s%s' % ('leakage',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __graphics_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__graphics_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'both'
        description = 'Display graphics at each stage to [screen|file|both|none]'
        value = self.__graphics( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'graphics': value},{'graphics': self.__schema['graphics']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-18.18s = %s%-23s%s' % ('graphics',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __verbose_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__verbose_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Print extra information to the logger and terminal'
        value = self.__verbose( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'verbose': value},{'verbose': self.__schema['verbose']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-18.18s = %s%-23s%s' % ('verbose',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __overwrite_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__overwrite_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return True
        description = 'Overwrite existing files in the project subdirectory'
        value = self.__overwrite( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'overwrite': value},{'overwrite': self.__schema['overwrite']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-18.18s = %s%-23s%s' % ('overwrite',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'sdant' in glb: del glb['sdant']
        if 'hourangle' in glb: del glb['hourangle']
        if 'totaltime' in glb: del glb['totaltime']
        if 'obsmode' in glb: del glb['obsmode']
        if 'pointingspacing' in glb: del glb['pointingspacing']
        if 'refdate' in glb: del glb['refdate']
        if 'complist' in glb: del glb['complist']
        if 'ptgfile' in glb: del glb['ptgfile']
        if 'incenter' in glb: del glb['incenter']
        if 'seed' in glb: del glb['seed']
        if 'verbose' in glb: del glb['verbose']
        if 'integration' in glb: del glb['integration']
        if 'project' in glb: del glb['project']
        if 'calflux' in glb: del glb['calflux']
        if 'user_pwv' in glb: del glb['user_pwv']
        if 'direction' in glb: del glb['direction']
        if 'compwidth' in glb: del glb['compwidth']
        if 'maptype' in glb: del glb['maptype']
        if 'incell' in glb: del glb['incell']
        if 'indirection' in glb: del glb['indirection']
        if 'mapsize' in glb: del glb['mapsize']
        if 'inbright' in glb: del glb['inbright']
        if 'thermalnoise' in glb: del glb['thermalnoise']
        if 'inwidth' in glb: del glb['inwidth']
        if 'leakage' in glb: del glb['leakage']
        if 'outframe' in glb: del glb['outframe']
        if 'antennalist' in glb: del glb['antennalist']
        if 'sdantlist' in glb: del glb['sdantlist']
        if 'overwrite' in glb: del glb['overwrite']
        if 'setpointings' in glb: del glb['setpointings']
        if 'caldirection' in glb: del glb['caldirection']
        if 't_ground' in glb: del glb['t_ground']
        if 'graphics' in glb: del glb['graphics']
        if 'tau0' in glb: del glb['tau0']
        if 't_sky' in glb: del glb['t_sky']
        if 'skymodel' in glb: del glb['skymodel']
        if 'comp_nchan' in glb: del glb['comp_nchan']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# simobserve -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__project_inp( )
        self.__skymodel_inp( )
        self.__inbright_inp( )
        self.__indirection_inp( )
        self.__incell_inp( )
        self.__incenter_inp( )
        self.__inwidth_inp( )
        self.__complist_inp( )
        self.__compwidth_inp( )
        self.__comp_nchan_inp( )
        self.__setpointings_inp( )
        self.__ptgfile_inp( )
        self.__integration_inp( )
        self.__direction_inp( )
        self.__mapsize_inp( )
        self.__maptype_inp( )
        self.__pointingspacing_inp( )
        self.__caldirection_inp( )
        self.__calflux_inp( )
        self.__obsmode_inp( )
        self.__refdate_inp( )
        self.__hourangle_inp( )
        self.__totaltime_inp( )
        self.__antennalist_inp( )
        self.__sdantlist_inp( )
        self.__sdant_inp( )
        self.__outframe_inp( )
        self.__thermalnoise_inp( )
        self.__user_pwv_inp( )
        self.__t_ground_inp( )
        self.__t_sky_inp( )
        self.__tau0_inp( )
        self.__seed_inp( )
        self.__leakage_inp( )
        self.__graphics_inp( )
        self.__verbose_inp( )
        self.__overwrite_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,savefile=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = savefile
        if filename is None:
            filename = "simobserve.last" if os.path.isfile("simobserve.last") else "simobserve.saved"
        if os.path.isfile(filename):
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file: %s\nsetting defaults instead..." % filename)
            self.set_global_defaults( )

    #--------- tput function ----------------------------------------------------------
    def tput(self,outfile=None):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s

        _postfile = outfile if outfile is not None else os.path.realpath('simobserve.last')

        _invocation_parameters = OrderedDict( )
        _invocation_parameters['project'] = self.__project( self.__globals_( ) )
        _invocation_parameters['skymodel'] = self.__skymodel( self.__globals_( ) )
        _invocation_parameters['inbright'] = self.__inbright( self.__globals_( ) )
        _invocation_parameters['indirection'] = self.__indirection( self.__globals_( ) )
        _invocation_parameters['incell'] = self.__incell( self.__globals_( ) )
        _invocation_parameters['incenter'] = self.__incenter( self.__globals_( ) )
        _invocation_parameters['inwidth'] = self.__inwidth( self.__globals_( ) )
        _invocation_parameters['complist'] = self.__complist( self.__globals_( ) )
        _invocation_parameters['compwidth'] = self.__compwidth( self.__globals_( ) )
        _invocation_parameters['comp_nchan'] = self.__comp_nchan( self.__globals_( ) )
        _invocation_parameters['setpointings'] = self.__setpointings( self.__globals_( ) )
        _invocation_parameters['ptgfile'] = self.__ptgfile( self.__globals_( ) )
        _invocation_parameters['integration'] = self.__integration( self.__globals_( ) )
        _invocation_parameters['direction'] = self.__direction( self.__globals_( ) )
        _invocation_parameters['mapsize'] = self.__mapsize( self.__globals_( ) )
        _invocation_parameters['maptype'] = self.__maptype( self.__globals_( ) )
        _invocation_parameters['pointingspacing'] = self.__pointingspacing( self.__globals_( ) )
        _invocation_parameters['caldirection'] = self.__caldirection( self.__globals_( ) )
        _invocation_parameters['calflux'] = self.__calflux( self.__globals_( ) )
        _invocation_parameters['obsmode'] = self.__obsmode( self.__globals_( ) )
        _invocation_parameters['refdate'] = self.__refdate( self.__globals_( ) )
        _invocation_parameters['hourangle'] = self.__hourangle( self.__globals_( ) )
        _invocation_parameters['totaltime'] = self.__totaltime( self.__globals_( ) )
        _invocation_parameters['antennalist'] = self.__antennalist( self.__globals_( ) )
        _invocation_parameters['sdantlist'] = self.__sdantlist( self.__globals_( ) )
        _invocation_parameters['sdant'] = self.__sdant( self.__globals_( ) )
        _invocation_parameters['outframe'] = self.__outframe( self.__globals_( ) )
        _invocation_parameters['thermalnoise'] = self.__thermalnoise( self.__globals_( ) )
        _invocation_parameters['user_pwv'] = self.__user_pwv( self.__globals_( ) )
        _invocation_parameters['t_ground'] = self.__t_ground( self.__globals_( ) )
        _invocation_parameters['t_sky'] = self.__t_sky( self.__globals_( ) )
        _invocation_parameters['tau0'] = self.__tau0( self.__globals_( ) )
        _invocation_parameters['seed'] = self.__seed( self.__globals_( ) )
        _invocation_parameters['leakage'] = self.__leakage( self.__globals_( ) )
        _invocation_parameters['graphics'] = self.__graphics( self.__globals_( ) )
        _invocation_parameters['verbose'] = self.__verbose( self.__globals_( ) )
        _invocation_parameters['overwrite'] = self.__overwrite( self.__globals_( ) )

        try:
            with open(_postfile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-15s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#simobserve( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: return False
        return True

    def __call__( self, project=None, skymodel=None, inbright=None, indirection=None, incell=None, incenter=None, inwidth=None, complist=None, compwidth=None, comp_nchan=None, setpointings=None, ptgfile=None, integration=None, direction=None, mapsize=None, maptype=None, pointingspacing=None, caldirection=None, calflux=None, obsmode=None, refdate=None, hourangle=None, totaltime=None, antennalist=None, sdantlist=None, sdant=None, outframe=None, thermalnoise=None, user_pwv=None, t_ground=None, t_sky=None, tau0=None, seed=None, leakage=None, graphics=None, verbose=None, overwrite=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('simobserve.pre')
        _postfile = os.path.realpath('simobserve.last')
        task_result = None
        _arguments = [project,skymodel,inbright,indirection,incell,incenter,inwidth,complist,compwidth,comp_nchan,setpointings,ptgfile,integration,direction,mapsize,maptype,pointingspacing,caldirection,calflux,obsmode,refdate,hourangle,totaltime,antennalist,sdantlist,sdant,outframe,thermalnoise,user_pwv,t_ground,t_sky,tau0,seed,leakage,graphics,verbose,overwrite]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if project is not None: local_global['project'] = project
            if skymodel is not None: local_global['skymodel'] = skymodel
            if complist is not None: local_global['complist'] = complist
            if setpointings is not None: local_global['setpointings'] = setpointings
            if obsmode is not None: local_global['obsmode'] = obsmode
            if outframe is not None: local_global['outframe'] = outframe
            if thermalnoise is not None: local_global['thermalnoise'] = thermalnoise
            if leakage is not None: local_global['leakage'] = leakage
            if graphics is not None: local_global['graphics'] = graphics
            if verbose is not None: local_global['verbose'] = verbose
            if overwrite is not None: local_global['overwrite'] = overwrite

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['project'] = self.__project( local_global )
            _invocation_parameters['skymodel'] = self.__skymodel( local_global )
            _invocation_parameters['complist'] = self.__complist( local_global )
            _invocation_parameters['setpointings'] = self.__setpointings( local_global )
            _invocation_parameters['obsmode'] = self.__obsmode( local_global )
            _invocation_parameters['outframe'] = self.__outframe( local_global )
            _invocation_parameters['thermalnoise'] = self.__thermalnoise( local_global )
            _invocation_parameters['leakage'] = self.__leakage( local_global )
            _invocation_parameters['graphics'] = self.__graphics( local_global )
            _invocation_parameters['verbose'] = self.__verbose( local_global )
            _invocation_parameters['overwrite'] = self.__overwrite( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            _invocation_parameters['inbright'] = self.__inbright( _invocation_parameters ) if inbright is None else inbright
            _invocation_parameters['indirection'] = self.__indirection( _invocation_parameters ) if indirection is None else indirection
            _invocation_parameters['incell'] = self.__incell( _invocation_parameters ) if incell is None else incell
            _invocation_parameters['incenter'] = self.__incenter( _invocation_parameters ) if incenter is None else incenter
            _invocation_parameters['inwidth'] = self.__inwidth( _invocation_parameters ) if inwidth is None else inwidth
            _invocation_parameters['compwidth'] = self.__compwidth( _invocation_parameters ) if compwidth is None else compwidth
            _invocation_parameters['comp_nchan'] = self.__comp_nchan( _invocation_parameters ) if comp_nchan is None else comp_nchan
            _invocation_parameters['ptgfile'] = self.__ptgfile( _invocation_parameters ) if ptgfile is None else ptgfile
            _invocation_parameters['integration'] = self.__integration( _invocation_parameters ) if integration is None else integration
            _invocation_parameters['direction'] = self.__direction( _invocation_parameters ) if direction is None else direction
            _invocation_parameters['mapsize'] = self.__mapsize( _invocation_parameters ) if mapsize is None else mapsize
            _invocation_parameters['maptype'] = self.__maptype( _invocation_parameters ) if maptype is None else maptype
            _invocation_parameters['pointingspacing'] = self.__pointingspacing( _invocation_parameters ) if pointingspacing is None else pointingspacing
            _invocation_parameters['caldirection'] = self.__caldirection( _invocation_parameters ) if caldirection is None else caldirection
            _invocation_parameters['calflux'] = self.__calflux( _invocation_parameters ) if calflux is None else calflux
            _invocation_parameters['refdate'] = self.__refdate( _invocation_parameters ) if refdate is None else refdate
            _invocation_parameters['hourangle'] = self.__hourangle( _invocation_parameters ) if hourangle is None else hourangle
            _invocation_parameters['totaltime'] = self.__totaltime( _invocation_parameters ) if totaltime is None else totaltime
            _invocation_parameters['antennalist'] = self.__antennalist( _invocation_parameters ) if antennalist is None else antennalist
            _invocation_parameters['sdantlist'] = self.__sdantlist( _invocation_parameters ) if sdantlist is None else sdantlist
            _invocation_parameters['sdant'] = self.__sdant( _invocation_parameters ) if sdant is None else sdant
            _invocation_parameters['user_pwv'] = self.__user_pwv( _invocation_parameters ) if user_pwv is None else user_pwv
            _invocation_parameters['t_ground'] = self.__t_ground( _invocation_parameters ) if t_ground is None else t_ground
            _invocation_parameters['t_sky'] = self.__t_sky( _invocation_parameters ) if t_sky is None else t_sky
            _invocation_parameters['tau0'] = self.__tau0( _invocation_parameters ) if tau0 is None else tau0
            _invocation_parameters['seed'] = self.__seed( _invocation_parameters ) if seed is None else seed

        else:
            # invoke with inp/go semantics
            _invocation_parameters['project'] = self.__project( self.__globals_( ) )
            _invocation_parameters['skymodel'] = self.__skymodel( self.__globals_( ) )
            _invocation_parameters['inbright'] = self.__inbright( self.__globals_( ) )
            _invocation_parameters['indirection'] = self.__indirection( self.__globals_( ) )
            _invocation_parameters['incell'] = self.__incell( self.__globals_( ) )
            _invocation_parameters['incenter'] = self.__incenter( self.__globals_( ) )
            _invocation_parameters['inwidth'] = self.__inwidth( self.__globals_( ) )
            _invocation_parameters['complist'] = self.__complist( self.__globals_( ) )
            _invocation_parameters['compwidth'] = self.__compwidth( self.__globals_( ) )
            _invocation_parameters['comp_nchan'] = self.__comp_nchan( self.__globals_( ) )
            _invocation_parameters['setpointings'] = self.__setpointings( self.__globals_( ) )
            _invocation_parameters['ptgfile'] = self.__ptgfile( self.__globals_( ) )
            _invocation_parameters['integration'] = self.__integration( self.__globals_( ) )
            _invocation_parameters['direction'] = self.__direction( self.__globals_( ) )
            _invocation_parameters['mapsize'] = self.__mapsize( self.__globals_( ) )
            _invocation_parameters['maptype'] = self.__maptype( self.__globals_( ) )
            _invocation_parameters['pointingspacing'] = self.__pointingspacing( self.__globals_( ) )
            _invocation_parameters['caldirection'] = self.__caldirection( self.__globals_( ) )
            _invocation_parameters['calflux'] = self.__calflux( self.__globals_( ) )
            _invocation_parameters['obsmode'] = self.__obsmode( self.__globals_( ) )
            _invocation_parameters['refdate'] = self.__refdate( self.__globals_( ) )
            _invocation_parameters['hourangle'] = self.__hourangle( self.__globals_( ) )
            _invocation_parameters['totaltime'] = self.__totaltime( self.__globals_( ) )
            _invocation_parameters['antennalist'] = self.__antennalist( self.__globals_( ) )
            _invocation_parameters['sdantlist'] = self.__sdantlist( self.__globals_( ) )
            _invocation_parameters['sdant'] = self.__sdant( self.__globals_( ) )
            _invocation_parameters['outframe'] = self.__outframe( self.__globals_( ) )
            _invocation_parameters['thermalnoise'] = self.__thermalnoise( self.__globals_( ) )
            _invocation_parameters['user_pwv'] = self.__user_pwv( self.__globals_( ) )
            _invocation_parameters['t_ground'] = self.__t_ground( self.__globals_( ) )
            _invocation_parameters['t_sky'] = self.__t_sky( self.__globals_( ) )
            _invocation_parameters['tau0'] = self.__tau0( self.__globals_( ) )
            _invocation_parameters['seed'] = self.__seed( self.__globals_( ) )
            _invocation_parameters['leakage'] = self.__leakage( self.__globals_( ) )
            _invocation_parameters['graphics'] = self.__graphics( self.__globals_( ) )
            _invocation_parameters['verbose'] = self.__verbose( self.__globals_( ) )
            _invocation_parameters['overwrite'] = self.__overwrite( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-15s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#simobserve( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _logging_state_ = None
            assert _pc.validate(_invocation_parameters,self.__schema), create_error_string(_pc.errors)
            _logging_state_ = _start_log( 'simobserve', [ 'project=' + repr(_pc.document['project']), 'skymodel=' + repr(_pc.document['skymodel']), 'inbright=' + repr(_pc.document['inbright']), 'indirection=' + repr(_pc.document['indirection']), 'incell=' + repr(_pc.document['incell']), 'incenter=' + repr(_pc.document['incenter']), 'inwidth=' + repr(_pc.document['inwidth']), 'complist=' + repr(_pc.document['complist']), 'compwidth=' + repr(_pc.document['compwidth']), 'comp_nchan=' + repr(_pc.document['comp_nchan']), 'setpointings=' + repr(_pc.document['setpointings']), 'ptgfile=' + repr(_pc.document['ptgfile']), 'integration=' + repr(_pc.document['integration']), 'direction=' + repr(_pc.document['direction']), 'mapsize=' + repr(_pc.document['mapsize']), 'maptype=' + repr(_pc.document['maptype']), 'pointingspacing=' + repr(_pc.document['pointingspacing']), 'caldirection=' + repr(_pc.document['caldirection']), 'calflux=' + repr(_pc.document['calflux']), 'obsmode=' + repr(_pc.document['obsmode']), 'refdate=' + repr(_pc.document['refdate']), 'hourangle=' + repr(_pc.document['hourangle']), 'totaltime=' + repr(_pc.document['totaltime']), 'antennalist=' + repr(_pc.document['antennalist']), 'sdantlist=' + repr(_pc.document['sdantlist']), 'sdant=' + repr(_pc.document['sdant']), 'outframe=' + repr(_pc.document['outframe']), 'thermalnoise=' + repr(_pc.document['thermalnoise']), 'user_pwv=' + repr(_pc.document['user_pwv']), 't_ground=' + repr(_pc.document['t_ground']), 't_sky=' + repr(_pc.document['t_sky']), 'tau0=' + repr(_pc.document['tau0']), 'seed=' + repr(_pc.document['seed']), 'leakage=' + repr(_pc.document['leakage']), 'graphics=' + repr(_pc.document['graphics']), 'verbose=' + repr(_pc.document['verbose']), 'overwrite=' + repr(_pc.document['overwrite']) ] )
            task_result = _simobserve_t( _pc.document['project'],_pc.document['skymodel'],_pc.document['inbright'],_pc.document['indirection'],_pc.document['incell'],_pc.document['incenter'],_pc.document['inwidth'],_pc.document['complist'],_pc.document['compwidth'],_pc.document['comp_nchan'],_pc.document['setpointings'],_pc.document['ptgfile'],_pc.document['integration'],_pc.document['direction'],_pc.document['mapsize'],_pc.document['maptype'],_pc.document['pointingspacing'],_pc.document['caldirection'],_pc.document['calflux'],_pc.document['obsmode'],_pc.document['refdate'],_pc.document['hourangle'],_pc.document['totaltime'],_pc.document['antennalist'],_pc.document['sdantlist'],_pc.document['sdant'],_pc.document['outframe'],_pc.document['thermalnoise'],_pc.document['user_pwv'],_pc.document['t_ground'],_pc.document['t_sky'],_pc.document['tau0'],_pc.document['seed'],_pc.document['leakage'],_pc.document['graphics'],_pc.document['verbose'],_pc.document['overwrite'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('simobserve')
            casalog.post("Exception Reported: Error in simobserve: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            raise #exception is now raised
            #task_result = False
        finally:
            try:
                os.rename(_prefile,_postfile)
            except: pass
            if _logging_state_:
                task_result = _end_log( _logging_state_, 'simobserve', task_result )

        #Added if _simobserve_t returns False and does not raise an exception.
        if task_result is False:
            raise

        return task_result #Still needed

simobserve = _simobserve( )

